@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * PMain - main module (loads and inits all startup modules)
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -96,6 +110,7 @@ static int cmdhlp(void)
 		printf("     curses       : ncurses driver\n");
 		printf("     x11          : x11 driver\n");
 		printf("     vcsa         : vcsa/fb linux console driver\n");
+		printf("     sdl          : SDL video driverr\n");
 		printf("\nExample : ocp -fl0,r1 -vp75,f2 -spdevpdisk -sr48000 ftstar.xm\n");
 		printf("          (for nice HD rendering of modules)\n");
 		return errHelpPrinted;
@@ -180,9 +195,6 @@ static int init_modules(int argc, char *argv[])
 			fprintf(stderr, "ocp.ini update (0.1.13/0.1.14) adds [devpCA]\n");
 			cfSetProfileString("devpCA", "link", "devpcoreaudio");
 
-			fprintf(stderr, "ocp.ini update (0.1.14) changed [devpOSS] revstereo to off\nn");
-			cfSetProfileBool("devpOSS", "revstereo", 0);
-
 			fprintf(stderr, "ocp.ini update (0.1.14) changed [devsOSS] revstereo to off\nn");
 			cfSetProfileBool("devsOSS", "revstereo", 0);	
 
@@ -215,9 +227,52 @@ static int init_modules(int argc, char *argv[])
 			cfSetProfileBool("x11", "xvidmode", cfGetProfileBool("x11", "xvidmode", 1, 1));
 		}
 
-		if (epoch < 20090208)
+		if (epoch <= 20090208)
 		{
-			cfSetProfileInt("version", "epoch", 20090208, 10);
+			fprintf(stderr, "ocp.ini update (0.1.18) removes [driver] keep=1\n");
+			cfRemoveEntry("devpALSA", "keep");
+		}
+
+		if (epoch <= 20100515)
+		{
+			fprintf(stderr, "ocp.ini update (0.1.19) adds [filetype 39]\n");
+			cfSetProfileInt("filetype 39", "color", 6, 10);
+			cfSetProfileString("filetype 39", "name", "YM");
+			cfSetProfileString("filetype 39", "interface", "plOpenCP");
+			cfSetProfileString("filetype 39", "pllink", "playym");
+			cfSetProfileString("filetype 39", "player", "ymPlayer");
+		}
+
+		if (epoch <= 20100515)
+		{
+			const char *temp;
+			char temp1[1024];
+			char temp2[1024];
+
+			temp = cfGetProfileString("fileselector", "modextensions", "");
+
+			if (strstr(temp, " YM"))
+			{
+				snprintf(temp1, sizeof(temp1), "%s", temp);
+			} else {
+				fprintf(stderr, "ocp.ini update (0.1.19) adds YM to [fileselector] modextensions=....\n");
+				snprintf(temp1, sizeof(temp1), "%s YM", temp);
+			}
+
+			if (strstr(temp, " OGA"))
+			{
+				snprintf(temp2, sizeof(temp2), "%s", temp1);
+			} else {
+				fprintf(stderr, "ocp.ini update (0.1.19) adds OGA to [fileselector] modextensions=....\n");
+				snprintf(temp2, sizeof(temp2), "%s OGA", temp1);
+			}
+
+			cfSetProfileString("fileselector", "modextensions", temp2);
+		}
+
+		if (epoch < 20100517)
+		{
+			cfSetProfileInt("version", "epoch", 20100517, 10);
 			cfStoreConfig();
 			if (isatty(2))
 			{
@@ -227,15 +282,14 @@ static int init_modules(int argc, char *argv[])
 			}
 			sleep(5);
 		}
-
 	}
-	if (cfGetProfileInt("version", "epoch", 0, 10)!=20090208)
+	if (cfGetProfileInt("version", "epoch", 0, 10)!=20100517)
 	{
 		if (isatty(2))
 		{
-			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20090208\033[0m\n\n");
+			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20100515\033[0m\n\n");
 		} else {
-			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20090208\033[0m\n\n");
+			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20100515\033[0m\n\n");
 		}
 		sleep(5);
 	}
