@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * CPIface text mode channel display
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -88,7 +102,7 @@ static void drawchannels()
 				writestring(buf, 0, plMuteCh[i]?0x08:0x07, " ##:", 4);
 				writestring(buf, 0, 0x0F, sign, 1);
 				_writenum(buf, 1, plMuteCh[i]?0x08:0x07, i+1, 10, 2);
-				ChanDisplay(buf+4, (plChanWidth==80)?76:128, y+first);
+				ChanDisplay(buf+4, (plChanWidth<128)?76:128, y+first);
 			} else {
 				writestring(buf, 0, plMuteCh[i]?0x08:0x07, "     ##:", 8);
 				writestring(buf, 4, 0x0F, sign, 1);
