@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * DevIGen - devices system overhead
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -86,7 +100,7 @@ int deviReadDevices(const char *list, struct devinfonode **devs)
 
 		bypass=cfGetProfileBool(drvhand, "bypass", 0, 0);
 		n->ihandle=handlfn++;
-		n->keep=cfGetProfileBool(drvhand, "keep", 0, 0);
+		n->keep=dev->keep;
 		n->devinfo.port=cfGetProfileInt(drvhand, "port", -1, 16);
 		n->devinfo.port2=cfGetProfileInt(drvhand, "port2", -1, 16);
 /*		n->dev.irq=cfGetProfileInt(drvhand, "irq", -1, 10);
@@ -191,4 +205,4 @@ int deviReadDevices(const char *list, struct devinfonode **devs)
 	return 1;
 }
 
-DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"devi", "OpenCP Devices Auxiliary Routines (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"devi", "OpenCP Devices Auxiliary Routines (c) 1994-10 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
