@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * Sampler devices system
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -11,11 +25,11 @@
  */
 
 #include "config.h"
-#include "devisamp.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include "types.h"
+#include "devisamp.h"
 #include "boot/plinkman.h"
 #include "boot/psetting.h"
 #include "devigen.h"
@@ -28,12 +42,12 @@
 #include "stuff/compat.h"
 #include "stuff/err.h"
 
-int (*smpProcessKey)(unsigned short);
+int (*smpProcessKey)(uint16_t);
 
 unsigned char plsmpOpt;
 unsigned short plsmpRate;
 
-struct devinfonode *plSamplerDevices;
+static struct devinfonode *plSamplerDevices;
 static struct devinfonode *cursampdev;
 static struct devinfonode *defsampdev;
 static struct mdbreaddirregstruct smpReadDirReg;
@@ -141,14 +155,14 @@ static void setdevice(struct devinfonode **curdev, struct devinfonode *dev)
 }
 
 
-void smpSetDevice(const char *name, int def)
+static void smpSetDevice(const char *name, int def)
 {
 	setdevice(&cursampdev, getdevstr(plSamplerDevices, name));
 	if (def)
 		defsampdev=cursampdev;
 }
 
-void smpResetDevice(void)
+static void smpResetDevice(void)
 {
 	setdevice(&cursampdev, defsampdev);
 }
@@ -324,4 +338,4 @@ static struct preprocregstruct smpPreprocess = {smpPrep PREPROCREGSTRUCT_TAIL};
 #ifndef SUPPORT_STATIC_PLUGINS
 char *dllinfo = "";
 #endif
-DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"smpbase", "OpenCP Sampler Devices System (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: sampdevinit, Close: sampdevclose};
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"smpbase", "OpenCP Sampler Devices System (c) 1994-10 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, Init: sampdevinit, Close: sampdevclose};
