@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * Sample processing routines (compression, mixer preparation etc)
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -183,14 +197,14 @@ static void repairloop(struct sampleinfo *s)
 static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 {
 	unsigned int newlen=s->length;
-	unsigned int replen=s->loopend-s->loopstart;
-	unsigned int sreplen=s->sloopend-s->sloopstart;
+	signed int replen=s->loopend-s->loopstart;
+	signed int sreplen=s->sloopend-s->sloopstart;
 	int restricted=0; /* boolean */
 	int toforward=0; /* boolean */
 	int stoforward=0; /* boolean */
 	unsigned int expandloop=0;
 	unsigned int sexpandloop=0;
-	unsigned int c;
+	signed int c;
 	void *newptr;
 
 #ifdef MCP_DEBUG
@@ -238,7 +252,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 
 	newptr=realloc(s->ptr, (newlen+SAMPEND)<<sampsizefac(s->type));
 #ifdef MCP_DEBUG
-	fprintf(stderr, __FILE__ ": realloced buffer=%p\n", newptr);
+	fprintf(stderr, __FILE__ ": realloced buffer=%p (size=%d)\n", newptr, (newlen+SAMPEND)<<sampsizefac(s->type));
 #endif
 	if (!newptr)
 	{
@@ -256,10 +270,11 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 		fprintf(stderr, __FILE__ ": replen=%d\n", replen);
 		fprintf(stderr, __FILE__ ": mcpSampBiDi set? %d\n", s->type&mcpSampBiDi);
 #endif
+
 		if (sampsizefac(s->type)==2)
 		{
 			float *p=(float *)s->ptr+s->loopend;
-			for (c=s->length-s->loopend-1; (signed)c>=0; c--)
+			for (c=s->length-s->loopend-1; c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
 				for (c=0; c<replen; c++)
@@ -272,7 +287,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 		} else if (sampsizefac(s->type)==1)
 		{
 			int16_t *p=(int16_t *)s->ptr+s->loopend;
-			for (c=s->length-s->loopend-1; (signed)c>=0; c--)
+			for (c=s->length-s->loopend-1; c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
 				for (c=0; c<replen; c++)
@@ -284,7 +299,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 				p[c]=p[c-(replen<<1)];
 		} else {
 			int8_t *p=(int8_t *)s->ptr+s->loopend;
-			for (c=s->length-s->loopend-1; (signed)c>=0; c--)
+			for (c=s->length-s->loopend-1; c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
 				for (c=0; c<replen; c++)
@@ -776,6 +791,47 @@ int mcpReduceSamples(struct sampleinfo *si, int n, long mem, int opt)
 
 #ifdef MCP_DEBUG
 	fprintf(stderr, __FILE__ ": mcpReduceSamples STARTED\n");
+	for (i=0; i<samplenum; i++)
+	{
+		fprintf(stderr, "type[%d]", i);
+		if (si[i].type & mcpSampUnsigned)
+			fprintf(stderr, " unsigned");
+		else
+			fprintf(stderr, " signed");
+		if (si[i].type & mcpSampDelta)
+			fprintf(stderr, " delta");
+		if (si[i].type & mcpSampFloat)
+			fprintf(stderr, " float");
+		else if (si[i].type & mcpSamp16Bit)
+			fprintf(stderr, " 16bit");
+		else
+			fprintf(stderr, " 8bit");
+		if (si[i].type & mcpSampBigEndian)
+			fprintf(stderr, " bigendian");
+		else
+			fprintf(stderr, " littleendian");
+		if (si[i].type & mcpSampBiDi)
+			fprintf(stderr, " bidi-loop");
+		else if (si[i].type & mcpSampLoop)
+			fprintf(stderr, " loop");
+		if (si[i].type & mcpSampSBiDi)
+			fprintf(stderr, " sbidi-loop");
+		else if (si[i].type & mcpSampSLoop)
+			fprintf(stderr, " sloop");
+		if (si[i].type & mcpSampStereo)
+			fprintf(stderr, " stereo");
+		else
+			fprintf(stderr, " mono");
+		fprintf(stderr, "\n");
+		fprintf(stderr, "ptr=%p\n", si[i].ptr);
+		fprintf(stderr, "length=%d\n", (int)si[i].length);
+		fprintf(stderr, "samprate=%d\n", (int)si[i].samprate);
+		fprintf(stderr, "loopstart=%d\n", (int)si[i].loopstart);
+		fprintf(stderr, "loopend=%d\n", (int)si[i].loopend);
+		fprintf(stderr, "sloopstart=%d\n", (int)si[i].sloopstart);
+		fprintf(stderr, "sloopend=%d\n", (int)si[i].sloopend);
+		fprintf(stderr, "\n\n");
+	}
 #endif
 	for (i=0; i<samplenum; i++)
 	{
