@@ -1,8 +1,22 @@
 /** OpenCP Module Player
- * copyright (c) '94-'05 Stian Skjelstad <stian@nixia.no>
+ * copyright (c) '94-'10 Stian Skjelstad <stian@nixia.no>
  *
  * ALSA (Advanced Linux Sound Architecture) Player device
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  *  revision history: (please note changes here)
  */
 
@@ -32,6 +46,11 @@
 static snd_pcm_t *alsa_pcm = NULL;
 static snd_mixer_t *mixer = NULL;
 
+static snd_pcm_status_t *alsa_pcm_status=NULL;
+static snd_pcm_info_t *pcm_info = NULL;
+static snd_pcm_hw_params_t *hwparams = NULL;
+static snd_pcm_sw_params_t *swparams = NULL;
+
 struct sounddevice plrAlsa;
 /*static struct deviceinfo currentcard;*/
 static struct mdbreaddirregstruct readdirAlsa;
@@ -50,7 +69,6 @@ static volatile int kernpos, cachepos, bufpos; /* in bytes */
 static volatile int cachelen, kernlen; /* to make life easier */
 static struct ocpvolstruct mixer_entries[MAX_ALSA_MIXER];
 static int alsa_mixers_n=0;
-
 /*  playbuf     kernpos  cachepos   bufpos      buflen
  *    |           | kernlen | cachelen |          |
  *
@@ -126,10 +144,8 @@ static int getplaypos(void)
 	{
 	} else {
 		snd_pcm_sframes_t tmp;	
-		snd_pcm_status_t *alsa_pcm_status=NULL;
 		int err;
 		
-		snd_pcm_status_alloca(&alsa_pcm_status);
 #ifdef ALSA_DEBUG
 		fprintf(stderr, "ALSA snd_pcm_status(alsa_pcm, alsa_pcm_status) = ");
 #endif
@@ -148,10 +164,17 @@ static int getplaypos(void)
 #ifdef ALSA_DEBUG
 			fprintf(stderr, "%ld\n", tmp);
 #endif
-			tmp*=1<<(bit16+stereo);
+			tmp<<=(bit16+stereo);
 	
 			if (tmp<0) /* we ignore buffer-underruns */
 				tmp=0;
+			else if (tmp==0)
+			{
+			/* ALSA sometimes (atlast on Stians Ubuntu laptop) gives odelay==0 always */
+				tmp = snd_pcm_status_get_avail_max(alsa_pcm_status) - snd_pcm_status_get_avail(alsa_pcm_status);
+				if (tmp<0)
+					tmp=0;
+			}
 		
 			if (tmp>kernlen)
 			{
@@ -169,7 +192,6 @@ static int getplaypos(void)
 static void flush(void)
 {
 	int result, n, odelay;
-	snd_pcm_status_t *alsa_pcm_status=NULL;
 	int err;
 
 	if (busy++)
@@ -177,9 +199,7 @@ static void flush(void)
 		busy--;
 		return;		
 	}
-
 		
-	snd_pcm_status_alloca(&alsa_pcm_status);
 #ifdef ALSA_DEBUG
 	fprintf(stderr, "ALSA snd_pcm_status(alsa_pcm, alsa_pcm_status) = ");
 #endif
@@ -200,9 +220,16 @@ static void flush(void)
 #ifdef ALSA_DEBUG
 	fprintf(stderr, "%i\n", odelay);
 #endif
-	odelay*=1<<(bit16+stereo);	
+	odelay<<=(bit16+stereo);	
 	if (odelay<0) /* we ignore buffer-underruns */
 		odelay=0;
+	else if (odelay==0)
+	{
+	/* ALSA sometimes (atlast on Stians Ubuntu laptop) gives odelay==0 always */
+		odelay = snd_pcm_status_get_avail_max(alsa_pcm_status) - snd_pcm_status_get_avail(alsa_pcm_status);
+		if (odelay<0)
+			odelay=0;
+	}
 
 	if (odelay>kernlen)
 	{
@@ -290,9 +317,7 @@ static uint32_t gettimer(void)
 	{
 		odelay=kernlen;
 	} else {
-		snd_pcm_status_t *alsa_pcm_status=NULL;
 		int err;
-		snd_pcm_status_alloca(&alsa_pcm_status);
 #ifdef ALSA_DEBUG
 		fprintf(stderr, "ALSA snd_pcm_status(alsa_pcm, alsa_pcm_status) = ");
 #endif
@@ -308,14 +333,22 @@ static uint32_t gettimer(void)
 			fprintf(stderr, "ok\n");
 			fprintf(stderr, "snd_pcm_status_get_delay(alsa_pcm_status) = ");
 #endif
+
 			odelay=snd_pcm_status_get_delay(alsa_pcm_status);
 #ifdef ALSA_DEBUG
 			fprintf(stderr, "%i\n", odelay);
 #endif
-			odelay*=1<<(bit16+stereo);	
 			if (odelay<0) /* we ignore buffer-underruns */
 				odelay=0;
+			else if (odelay==0)
+			{
+			/* ALSA sometimes (atlast on Stians Ubuntu laptop) gives odelay==0 always */
+				odelay = snd_pcm_status_get_avail_max(alsa_pcm_status) - snd_pcm_status_get_avail(alsa_pcm_status);
+				if (odelay<0)
+					odelay=0;
+			}
 
+			odelay<<=(bit16+stereo);
 			if (odelay>kernlen)
 			{
 				odelay=kernlen;
@@ -603,7 +636,6 @@ static int list_devices_for_card(int card, struct modlist *ml, const struct dmDr
 	int pcm_device=-1;
 	int cards=0;
 	snd_ctl_t *ctl;
-	snd_pcm_info_t *pcm_info;
 	
 	snprintf(dev, sizeof(dev), "hw:%i", card);
 #ifdef ALSA_DEBUG
@@ -631,11 +663,6 @@ static int list_devices_for_card(int card, struct modlist *ml, const struct dmDr
 	fprintf(stderr, "ok, %s name: %s\n", dev, card_name);
 #endif
 
-#ifdef ALSA_DEBUG
-	fprintf(stderr, "ALSA snd_pcm_info_alloca(&pcm_info)\n");
-#endif
-	snd_pcm_info_alloca(&pcm_info);
-	
 	while(1)
 	{
 #ifdef ALSA_DEBUG
@@ -877,8 +904,6 @@ static void SetOptions(unsigned int rate, int opt)
 {
 	int err;
 	snd_pcm_format_t format;
-	snd_pcm_hw_params_t *hwparams;
-	snd_pcm_sw_params_t *swparams;
 	unsigned int val;
 	/* start with setting default values, if we bail out */
 	plrRate=rate;
@@ -888,9 +913,6 @@ static void SetOptions(unsigned int rate, int opt)
 	if (!alsa_pcm)
 		return;
 
-	snd_pcm_hw_params_alloca(&hwparams);
-	snd_pcm_sw_params_alloca(&swparams);
-
 #ifdef ALSA_DEBUG
 	fprintf(stderr, "ALSA snd_pcm_hw_params_any(alsa_pcm, hwparams) = ");
 #endif
@@ -1463,7 +1485,28 @@ static int alsaReadDir(struct modlist *ml, const struct dmDrive *drive, const ui
 
 static void __attribute__((constructor))init(void)
 {
+	int err;
 	mdbRegisterReadDir(&readdirAlsa);
+	if ((err = snd_pcm_status_malloc(&alsa_pcm_status)))
+	{
+		fprintf(stderr, "snd_pcm_status_malloc() failed, %s\n", snd_strerror(-err));
+		exit(0);
+	}
+	if ((err = snd_pcm_info_malloc(&pcm_info)))
+	{
+		fprintf(stderr, "snd_pcm_info_malloc() failed, %s\n", snd_strerror(-err));
+		exit(0);
+	}
+	if ((err = snd_pcm_hw_params_malloc(&hwparams)))
+	{
+		fprintf(stderr, "snd_pcm_hw_params_malloc failed, %s\n", snd_strerror(-err));
+		exit(0);
+	}
+	if ((err = snd_pcm_sw_params_malloc(&swparams)))
+	{
+		fprintf(stderr, "snd_pcm_hw_params_malloc failed, %s\n", snd_strerror(-err));
+		exit(0);
+	}
 }
 
 static void __attribute__((destructor))fini(void)
@@ -1507,10 +1550,31 @@ static void __attribute__((destructor))fini(void)
 #endif
 		mixer=NULL;
 	}
+	if (alsa_pcm_status)
+	{
+		snd_pcm_status_free(alsa_pcm_status);
+		alsa_pcm_status = NULL;
+	}
+	if (pcm_info)
+	{
+		snd_pcm_info_free(pcm_info);
+		pcm_info = NULL;
+	}
+	if (hwparams)
+	{
+		snd_pcm_hw_params_free(hwparams);
+		hwparams=NULL;
+	}
+	if (swparams)
+	{
+		snd_pcm_sw_params_free(swparams);
+		swparams=NULL;
+	}
 }
 
-struct sounddevice plrAlsa={SS_PLAYER, "ALSA device driver", alsaDetect, alsaInit, alsaClose, NULL};
+struct sounddevice plrAlsa={SS_PLAYER, 1, "ALSA device driver", alsaDetect, alsaInit, alsaClose, NULL};
 struct ocpvolregstruct volalsa={volalsaGetNumVolume, volalsaGetVolume, volalsaSetVolume};
 static struct mdbreaddirregstruct readdirAlsa = {alsaReadDir MDBREADDIRREGSTRUCT_TAIL};
+
 char *dllinfo="driver plrAlsa; volregs volalsa";
-struct linkinfostruct dllextinfo = {"devpalsa", "OpenCP Player Device: ALSA (c) 2005-06 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+struct linkinfostruct dllextinfo = {"devpalsa", "OpenCP Player Device: ALSA (c) 2005-10 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
