@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * Player device for OSS output
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -ss040613   Stian Skjelstad <stian@nixia.no>
  *    -first release
@@ -601,10 +615,8 @@ static int ossDetect(struct deviceinfo *card)
 	if (ioctl(fd_dsp, OSS_GETVERSION, &tmp)<0)
 	{
 #ifdef OSS_DEBUG
-		perror("devposs: ioctl(fd_dsp, OSS_GETVERSION, &tmp)");
+		perror("devposs: (warning) ioctl(fd_dsp, OSS_GETVERSION, &tmp)");
 #endif
-		close(fd_dsp);
-		return 0;
 	}
 #endif
 
@@ -742,8 +754,8 @@ static int volossSetVolume(struct ocpvolstruct *v, int n)
 }
 
 static struct devaddstruct plrOSSAdd = {ossGetOpt, 0, 0, 0};
-struct sounddevice plrOSS={SS_PLAYER, "OSS player", ossDetect,  ossInit,  ossClose, &plrOSSAdd};
+struct sounddevice plrOSS={SS_PLAYER, 0, "OSS player", ossDetect,  ossInit,  ossClose, &plrOSSAdd};
 struct ocpvolregstruct voloss={volossGetNumVolume, volossGetVolume, volossSetVolume};
 
 char *dllinfo = "driver plrOSS; volregs voloss";
-struct linkinfostruct dllextinfo = {"devposs", "OpenCP Player Device: OSS (c) '04-'05 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+struct linkinfostruct dllextinfo = {"devposs", "OpenCP Player Device: OSS (c) 2004-09 Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
