@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * NoSound sampler device (samples perfect noise-free 16bit silence)
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -53,7 +67,7 @@ static int ndStart(unsigned char **buf, int  *len)
 	if (smpOpt&SMP_16BIT)
 		initval&=0xFF00FF00;
 
-	dmalen=*len>>((!!smpOpt&SMP_STEREO)+(!!smpOpt&SMP_16BIT));
+	dmalen=(*len)>>((!!(smpOpt&SMP_STEREO))+(!!(smpOpt&SMP_16BIT)));
 	memsetd(*buf, initval, dmalen);
 	
 	smpGetBufPos=getbufpos;
@@ -103,6 +117,6 @@ static int ndDetect(struct deviceinfo *c)
 }
 
 
-struct sounddevice smpNone={SS_SAMPLER, "Eternal Silence Recorder", ndDetect, ndInit, ndClose, 0};
+struct sounddevice smpNone={SS_SAMPLER, 0, "Eternal Silence Recorder", ndDetect, ndInit, ndClose, 0};
 char *dllinfo="driver smpNone";
-struct linkinfostruct dllextinfo = {"devsnone", "OpenCP Sampler Device: None (c) 1994-04 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+struct linkinfostruct dllextinfo = {"devsnone", "OpenCP Sampler Device: None (c) 1994-09 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
