@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * Wavetable Device: Software Mixer for sample stream output via devp
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -26,10 +40,6 @@
 #include <unistd.h>
 
 #include <sys/mman.h>
-#include <limits.h>    /* for PAGESIZE */
-#ifndef PAGESIZE
-#define PAGESIZE 4096
-#endif
 
 #include "types.h"
 #include "boot/plinkman.h"
@@ -44,6 +54,10 @@
 #include "dwmixa.h"
 #include "dwmixqa.h"
 
+#ifdef I386_ASM
+#include "stuff/pagesize.inc.c"
+#endif
+
 #define MIXBUFLEN 4096
 #define MAXCHAN 255
 
@@ -813,7 +827,7 @@ static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate
 		chn->samp=(void*)((unsigned long)c->samp<<1);
 	else
 		chn->samp=c->samp;
-	chn->realsamp.fmt=c->samp;
+	chn->realsamp.fmt=chn->samp; /* at this point, they match */
 	chn->length=c->length;
 	chn->loopstart=c->loopstart;
 	chn->loopend=c->loopend;
@@ -849,10 +863,10 @@ static int LoadSamples(struct sampleinfo *sil, int n)
 	
 	for (i=0;i<n;i++)
 	{
-		samples[i].ptr=malloc(samples[i].length+PAGESIZE*10);
-		samples[i].ptr= (char *)(((int) samples[i].ptr + PAGESIZE*4) & ~(PAGESIZE-1) );
+		samples[i].ptr=malloc(samples[i].length+pagesize()*10);
+		samples[i].ptr= (char *)(((int) samples[i].ptr + pagesize()*4) & ~(pagesize()-1) );
 		memcpy(samples[i].ptr, sil[i].ptr, samples[i].length);
-		mprotect((void *)((int)(samples[i].ptr) & ~(PAGESIZE-1)), samples[i].length, PROT_READ);
+		mprotect((void *)((int)(samples[i].ptr) & ~(pagesize()-1)), samples[i].length, PROT_READ);
 	}
 	samplenum=n;
 
@@ -1107,10 +1121,10 @@ static int wmixInit(const struct deviceinfo *dev)
 		fprintf(stderr, "range2: %p - %p\n", start2, stop2);
 #endif
 
-		start1=(char *)(((int)start1)&~(PAGESIZE-1));
-		start2=(char *)(((int)start2)&~(PAGESIZE-1));
-		len1=((stop1-start1)+PAGESIZE-1)& ~(PAGESIZE-1);
-		len2=((stop2-start2)+PAGESIZE-1)& ~(PAGESIZE-1);
+		start1=(char *)(((int)start1)&~(pagesize()-1));
+		start2=(char *)(((int)start2)&~(pagesize()-1));
+		len1=((stop1-start1)+pagesize()-1)& ~(pagesize()-1);
+		len2=((stop2-start2)+pagesize()-1)& ~(pagesize()-1);
 #ifdef MIXER_DEBUG
 		fprintf(stderr, "mprot: %p + %08x\n", start1, len1);
 		fprintf(stderr, "mprot: %p + %08x\n", start2, len2);
@@ -1142,8 +1156,8 @@ static int wmixInit(const struct deviceinfo *dev)
 			return 0;
 		}
 		
-/*	if (mprotect((char *)(((int)remap_range1_start)&~(PAGESIZE-1)), (((char *)remap_range1_stop-(char *)remap_range1_start)+PAGESIZE-1)& ~(PAGESIZE-1), PROT_READ|PROT_WRITE|PROT_EXEC) ||
-	    mprotect((char *)(((int)remap_range2_start)&~(PAGESIZE-1)), (((char *)remap_range2_stop-(char *)remap_range2_start)+PAGESIZE-1)& ~(PAGESIZE-1), PROT_READ|PROT_WRITE|PROT_EXEC) )
+/*	if (mprotect((char *)(((int)remap_range1_start)&~(pagesize()-1)), (((char *)remap_range1_stop-(char *)remap_range1_start)+pagesize()-1)& ~(pagesize()-1), PROT_READ|PROT_WRITE|PROT_EXEC) ||
+	    mprotect((char *)(((int)remap_range2_start)&~(pagesize()-1)), (((char *)remap_range2_stop-(char *)remap_range2_start)+pagesize()-1)& ~(pagesize()-1), PROT_READ|PROT_WRITE|PROT_EXEC) )
 	    {
 			perror("Couldn't mprotect"); 
  			return 0;
@@ -1208,6 +1222,27 @@ static void mixrInit(const char *sec)
 	char regname[50];
 	const char *regs;
 
+
+	fprintf(stderr, "[devwmix] INIT, ");
+	if (!quality)
+	{
+#ifdef I386_ASM 
+		fprintf(stderr, "using dwmixa.c x86-asm version\n");
+#else
+#ifdef I386_ASM_EMU
+		fprintf(stderr, "using dwmixa.c x86-emu-asm version\n");
+#else
+		fprintf(stderr, "using dwmixa.c C version\n");
+#endif
+#endif
+	} else {
+#ifdef I386_ASM 
+		fprintf(stderr, "using dwmixaq.c x86-asm version\n");
+#else
+		fprintf(stderr, "using dwmixaq.c C version\n");
+#endif
+	}
+
 	postprocs=0;
 	regs=cfGetProfileString(sec, "postprocs", "");
 
@@ -1248,7 +1283,7 @@ static int mixProcKey(unsigned short key)
 }
 
 struct devaddstruct mcpMixAdd = {mixGetOpt, mixrInit, 0, mixProcKey};
-struct sounddevice mcpMixer={SS_WAVETABLE|SS_NEEDPLAYER, "Mixer", wmixDetect, wmixInit, wmixClose, &mcpMixAdd};
+struct sounddevice mcpMixer={SS_WAVETABLE|SS_NEEDPLAYER, 0, "Mixer", wmixDetect, wmixInit, wmixClose, &mcpMixAdd};
 char *dllinfo="driver mcpMixer";
 
-struct linkinfostruct dllextinfo = {"devwmix", "OpenCP Wavetable Device: Mixer (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+struct linkinfostruct dllextinfo = {"devwmix", "OpenCP Wavetable Device: Mixer (c) 1994-10 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
