@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * Wavetable Device: FPU HighQuality Software Mixer for Pentium/above CPUs
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -kb990208   Tammo Hinrichs <kb@vmo.nettrade.de>
  *    -first release
@@ -47,6 +61,9 @@
 #include "devwmixf.h"
 #include "stuff/poll.h"
 #include "dwmixfa.h"
+#ifdef I386_ASM
+#include "stuff/pagesize.inc.c"
+#endif
 
 /* NB NB   more includes further down in the file */
 
@@ -76,8 +93,44 @@ static uint8_t signedout;
 static uint8_t reversestereo;
 
 static int     channelnum;
+
+struct channel
+{
+	void *samp;
+
+	uint32_t length;
+	uint32_t loopstart;
+	uint32_t loopend;
+
+	int   newsamp;
+
+	float dstvols[2];
+	int   dontramp;
+
+	float vol[2];
+	float orgvol[2];
+
+	float orgvolx;
+	float orgpan;
+	float orgfrez;
+
+	float *sbpos;
+	float sbuf[8];
+
+	uint32_t orgrate;
+	uint32_t orgfrq;
+	uint32_t orgdiv;
+	int      volopt;
+	uint32_t samptype;
+
+	uint32_t orgloopstart;
+	uint32_t orgloopend;
+	uint32_t orgsloopstart;
+	uint32_t orgsloopend;
+
+	long  handle;
+};
 static struct channel *channels;
-static float   fadedown[2];
 
 static unsigned int bufpos;
 static uint32_t buflen;
@@ -100,28 +153,22 @@ static int   masterchr;
 
 inline float frchk(float a)
 {
-#if 1
-#warning This is not portable
-	union {
-		float a;
-		uint32_t b;
-	} cast;
-	cast.a=a;
-	cast.b&=0x7f800000;
-	cast.b>>=23;
-	if (cast.b==0 || cast.b==0x000000ff)
-		return 0.0;
-#else
-#warning This is not portable
-	uint32_t b=*((uint32_t *)(void *)&a); /* makes compilers shut up warnings */
-	b&=0x7f800000;
-	b>>=23;
-	if (b==0 || b==0xff)
-		return 0.0;
-#endif
-	if (fabs(a)<0.00000001)
-		return 0.0;
-	return a;
+	switch (fpclassify(a))
+	{
+		default:
+		case FP_NAN:
+		case FP_INFINITE:
+		case FP_SUBNORMAL:
+
+		case FP_ZERO:
+			return 0.0f;
+
+		case FP_NORMAL:
+			if (fabs(a)<0.00000001)
+				return 0.0;
+			else
+				return a;
+	}
 }
 
 static void calcinterpoltab(void)
@@ -294,13 +341,6 @@ static void mixmain(/*int min*/)
 	int /*bufmin,*/
 	    bufdeltatot;
 
-  #ifdef RASTER
-	outp(0x3c8,0);
-	outp(0x3c9,63);
-	outp(0x3c9,0);
-	outp(0x3c9,0);
-  #endif
-
 	if (!channelnum)
 		return;
 	if (clipbusy++)
@@ -386,7 +426,7 @@ static void mixmain(/*int min*/)
 				ch->newsamp=0;
 			}
 
-			voiceflags[i]|=isstereo;
+			/*voiceflags[i]|=isstereo; <- this was hard to track down*/
 		}
 		nsamples=bufdelta;
 		outbuf=((uint8_t *)(plrbuf))+(bufpos<<(stereo+bit16));
@@ -406,7 +446,7 @@ static void mixmain(/*int min*/)
 					fprintf(stderr, "  looplen: %08x\n", looplen[i]);
 					fprintf(stderr, "  voiceflags: 0x%08x\n", voiceflags[i]);
 					fprintf(stderr, "  freq: 0x%08x:0x%08x\n", freqw[i], freqf[i]);
-					fprintf(stderr, "  vol %d %d\n", volleft[i], volright[i]);
+					fprintf(stderr, "  vol %lf %lf\n", volleft[i], volright[i]);
 				}
 			fprintf(stderr, "\n");
 		}
@@ -459,13 +499,6 @@ static void mixmain(/*int min*/)
 	}
 
 	clipbusy--;
-
-  #ifdef RASTER
-	outp(0x3c8,0);
-	outp(0x3c9,0);
-	outp(0x3c9,0);
-	outp(0x3c9,0);
-  #endif
 }
 
 static void timerproc()
@@ -781,7 +814,7 @@ static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate
 	chn->length=c->length;
 	chn->loopstart=c->loopstart;
 	chn->loopend=c->loopend;
-	chn->fpos=smpposf[ch];
+	chn->fpos=smpposf[ch]>>16;
 	chn->pos=smpposw[ch]-(float*)c->samp;
 	chn->vol.volfs[0]=fabs(c->vol[0]);
 	chn->vol.volfs[1]=fabs(c->vol[1]);
@@ -834,7 +867,6 @@ static int OpenPlayer(int chan, void (*proc)(void))
 	uint16_t mixfate;
 	int i;
 
-	fadedown[0]=fadedown[1]=0;
 	playsamps=pausesamps=0;
 	if (chan>MIXF_MAXCHAN)
 		chan=MIXF_MAXCHAN;
@@ -957,8 +989,8 @@ static int Init(const struct deviceinfo *dev)
 #ifdef MIXER_DEBUG
 		fprintf(stderr, "range1: %p - %p\n", start1, stop1);
 #endif
-		start1=(char *)(((int)start1)&~(PAGESIZE-1));
-		len1=((stop1-start1)+PAGESIZE-1)& ~(PAGESIZE-1);
+		start1=(char *)(((int)start1)&~(pagesize()-1));
+		len1=((stop1-start1)+pagesize()-1)& ~(pagesize()-1);
 #ifdef MIXER_DEBUG
 		fprintf(stderr, "mprot: %p + %08x\n", start1, len1);
 #endif
@@ -975,7 +1007,7 @@ static int Init(const struct deviceinfo *dev)
 			return 0;
 		}
 
-/*	if (mprotect((char *)(((int)remap_range1_start)&~(PAGESIZE-1)), (((char *)remap_range1_stop-(char *)remap_range1_start)+PAGESIZE-1)& ~(PAGESIZE-1), PROT_READ|PROT_WRITE|PROT_EXEC) )
+/*	if (mprotect((char *)(((int)remap_range1_start)&~(pagesize()-1)), (((char *)remap_range1_stop-(char *)remap_range1_start)+pagesize()-1)& ~(pagesize()-1), PROT_READ|PROT_WRITE|PROT_EXEC) )
 	    {
 			perror("Couldn't mprotect"); 
  			return 0;
@@ -1066,6 +1098,18 @@ static void mixfInit(const char *sec)
 	char regname[50];
 	const char *regs;
 
+	fprintf(stderr, "[devwmixf] INIT, ");
+#ifdef I386_ASM 
+		fprintf(stderr, "using dwmixfa.c x86-asm version\n");
+#else
+#ifdef I386_ASM_EMU
+		fprintf(stderr, "using dwmixfa.c x86-emu-asm version\n");
+#else
+		fprintf(stderr, "using dwmixfa.c C version\n");
+		/*fprintf(stderr, "using dwmixa.c C version\n");*/
+#endif
+#endif
+
 	postprocs=0;
 	regs=cfGetProfileString(sec, "postprocs", "");
 
@@ -1105,7 +1149,7 @@ static int mixfProcKey(uint16_t key)
 }
 
 struct devaddstruct mcpFMixAdd = {mixfGetOpt, mixfInit, 0, mixfProcKey};
-struct sounddevice mcpFMixer={SS_WAVETABLE|SS_NEEDPLAYER, "FPU Mixer", Detect, Init, Close, &mcpFMixAdd};
+struct sounddevice mcpFMixer={SS_WAVETABLE|SS_NEEDPLAYER, 0, "FPU Mixer", Detect, Init, Close, &mcpFMixAdd};
 char *dllinfo="driver mcpFMixer";
 
-struct linkinfostruct dllextinfo = {"devwmixf", "OpenCP Wavetable Device: FPU HighQuality Mixer (c) 1999-04 Tammo Hinrichs, Fabian Giesen", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+struct linkinfostruct dllextinfo = {"devwmixf", "OpenCP Wavetable Device: FPU HighQuality Mixer (c) 1999-10 Tammo Hinrichs, Fabian Giesen, Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
