@@ -1,7 +1,20 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * assembler routines for 8-bit MCP mixer
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
@@ -19,6 +32,12 @@
 
 #include "dev/mix.h"
 
+#ifndef I386_ASM
+#ifdef I386_ASM_EMU
+#include "asm_emu/x86.h"
+#endif
+#endif
+
 #ifdef I386_ASM
 
 #ifndef NULL
@@ -524,7 +543,7 @@ void mixrPlayChannel(int32_t *buf, int32_t *fadebuf, uint32_t len, struct channe
 		"    cmpl %%edx, %%ecx\n"
 		"    jbe mixrPlayChannelnoramp0\n"
 		"      movb $1, %10\n"            /* %10 = ramploop */
-		"    movl %%edx, %%ecx\n"
+		"      movl %%edx, %%ecx\n"
 		"mixrPlayChannelnoramp0:\n"
 
 		"  movl %22(%%edi), %%edx\n"      /* %22 = ch->dstvols[1] */
@@ -1080,18 +1099,18 @@ void mixrSetupAddresses(int32_t (*vol)[256], uint8_t (*intr)[256][2])
 }
 
 #include <stdio.h>
-void mixrFadeChannel(int32_t *fade, struct channel *ch)
+void mixrFadeChannel(int32_t *fade, struct channel *chan)
 {
-	if (ch->status&MIXRQ_PLAY16BIT)
+	if (chan->status&MIXRQ_PLAY16BIT)
 	{
-		fade[0]+=mixrFadeChannelvoltab[ch->curvols[0]][((uint16_t)ch->realsamp.bit16[ch->pos])>>8];
-		fade[1]+=mixrFadeChannelvoltab[ch->curvols[1]][((uint16_t)ch->realsamp.bit16[ch->pos])>>8];
+		fade[0]+=mixrFadeChannelvoltab[chan->curvols[0]][((uint16_t)chan->realsamp.bit16[chan->pos])>>8];
+		fade[1]+=mixrFadeChannelvoltab[chan->curvols[1]][((uint16_t)chan->realsamp.bit16[chan->pos])>>8];
 	} else {
-		fade[0]+=mixrFadeChannelvoltab[ch->curvols[0]][(uint8_t)ch->realsamp.bit8[ch->pos]];
-		fade[1]+=mixrFadeChannelvoltab[ch->curvols[1]][(uint8_t)ch->realsamp.bit8[ch->pos]];
+		fade[0]+=mixrFadeChannelvoltab[chan->curvols[0]][(uint8_t)chan->realsamp.bit8[chan->pos]];
+		fade[1]+=mixrFadeChannelvoltab[chan->curvols[1]][(uint8_t)chan->realsamp.bit8[chan->pos]];
 	}
-	ch->curvols[0]=0;
-	ch->curvols[1]=0;
+	chan->curvols[0]=0;
+	chan->curvols[1]=0;
 }
 
 
@@ -1318,6 +1337,372 @@ static const route_func routeptrs[8]=
 	playstereoi16
 };
 
+#ifdef I386_ASM_EMU
+static void writecallback(uint_fast16_t selector, uint_fast32_t addr, int size, uint_fast32_t data)
+{
+}
+
+static uint_fast32_t readcallback(uint_fast16_t selector, uint_fast32_t addr, int size)
+{
+	return 0;
+}
+
+void mixrPlayChannel(int32_t *buf, int32_t *fadebuf, uint32_t len, struct channel *chan, int stereo)
+{
+	int32_t *orgbuf = buf;
+	uint32_t orglen = len;
+
+	uint32_t filllen;
+/*	         ramping[2];*/
+	uint32_t inloop;
+	uint32_t ramploop;
+	uint32_t dofade;
+	route_func routeptr;
+
+	uint32_t step_high;
+	uint32_t step_low;
+	uint32_t ramping0;
+	uint32_t ramping1;
+	uint32_t endofbuffer;
+	uint32_t bufferlen;
+
+	struct assembler_state_t state;
+	init_assembler_state(&state, writecallback, readcallback);
+
+	asm_movl(&state, 0xf0000000, &state.edi); /* &chan == 0xf0000000   only dummy, since we won't deref chan directly */
+	asm_testb(&state, MIXRQ_PLAYING, chan->status);
+
+	asm_jz(&state, mixrPlayChannelexit);
+
+	asm_movl(&state, 0, (uint32_t *)&filllen);
+	asm_movl(&state, 0, (uint32_t *)&dofade);
+
+	asm_xorl(&state, state.eax, &state.eax);
+	asm_cmpl(&state, 0, stereo);
+	asm_je(&state, mixrPlayChannelnostereo);
+		asm_addl(&state, 4, &state.eax);
+mixrPlayChannelnostereo:
+	asm_testb(&state, MIXRQ_INTERPOLATE, chan->status);
+
+	asm_jz(&state, mixrPlayChannelnointr);
+		asm_addl(&state, 2, &state.eax);
+mixrPlayChannelnointr:
+	asm_testb(&state, MIXRQ_PLAY16BIT, chan->status);
+
+	asm_jz(&state, mixrPlayChannelpsetrtn);
+		asm_incl(&state, &state.eax);	
+mixrPlayChannelpsetrtn:
+	asm_shll(&state, 5, &state.eax);
+	asm_addl(&state, 0xf8000000, &state.eax);
+	{
+		unsigned int tmp = 0;
+		asm_movl(&state, state.eax, &tmp);
+		tmp-=0xf8000000;
+		tmp/=8; /* 8 columns of information in the assembler version */
+		if (tmp > sizeof(routeptrs))
+		{
+			fprintf(stderr, "#GP Exception occured (tmp=%d)\n", tmp);
+			tmp = 0;
+		}
+		else if (tmp&3)
+		{
+			fprintf(stderr, "#AC(0) Exception occured\n");
+			tmp = 0;
+		}
+		routeptr = routeptrs[tmp>>2];
+	}
+
+mixrPlayChannelbigloop:
+	asm_movl(&state, len, &state.ecx);
+	asm_movl(&state, chan->step, &state.ebx);
+	asm_movl(&state, chan->pos, &state.edx);
+	asm_movw(&state, chan->fpos, &state.si);
+	asm_movl(&state, 0, &inloop);
+	asm_cmpl(&state, 0, state.ebx);
+	
+	asm_je(&state, mixrPlayChannelplayecx);
+	asm_jg(&state, mixrPlayChannelforward);
+		asm_negl(&state, &state.ebx);
+		asm_movl(&state, state.edx, &state.eax);
+		asm_testb(&state, MIXRQ_LOOPED, chan->status);
+
+		asm_jz(&state, mixrPlayChannelmaxplaylen);
+		asm_cmpl(&state, chan->loopstart, state.edx);
+		asm_jb(&state, mixrPlayChannelmaxplaylen);
+		asm_subl(&state, chan->loopstart, &state.eax);
+		asm_movl(&state, 1, &inloop);
+		asm_jmp(&state, mixrPlayChannelmaxplaylen);
+mixrPlayChannelforward:
+		asm_movl(&state, chan->length, &state.eax);
+		asm_negw(&state, &state.si);
+		asm_sbbl(&state, state.edx, &state.eax);
+		asm_testb(&state, MIXRQ_LOOPED, chan->status);
+
+		asm_jz(&state, mixrPlayChannelmaxplaylen);
+		asm_cmpl(&state, chan->loopend, state.edx);
+		asm_jae(&state, mixrPlayChannelmaxplaylen);
+		asm_subl(&state, chan->length, &state.eax);
+		asm_addl(&state, chan->loopend, &state.eax);
+		asm_movl(&state, 1, &inloop);
+
+mixrPlayChannelmaxplaylen:
+	asm_xorl(&state, state.edx, &state.edx);
+	asm_shldl(&state, 16, state.eax, &state.edx);
+	asm_shll(&state, 16, &state.esi);
+	asm_shldl(&state, 16, state.esi, &state.eax);
+	asm_addl(&state, state.ebx, &state.eax);
+	asm_adcl(&state, 0, &state.edx);
+	asm_subl(&state, 1, &state.eax);
+	asm_sbbl(&state, 0, &state.edx);
+	asm_cmpl(&state, state.ebx, state.edx);
+	asm_jae(&state, mixrPlayChannelplayecx);
+	asm_divl(&state, state.ebx);
+	asm_cmpl(&state, state.eax, state.ecx);
+	asm_jb(&state, mixrPlayChannelplayecx);
+		asm_movl(&state, state.eax, &state.ecx);
+		asm_cmpl(&state, 0, inloop);
+		asm_jnz(&state, mixrPlayChannelplayecx);
+
+		asm_andw(&state, ~MIXRQ_PLAYING, (uint16_t *)&chan->status);
+
+		asm_movl(&state, 1, &dofade);
+		asm_movl(&state, len, &state.eax);
+		asm_subl(&state, state.ecx, &state.eax);
+		asm_addl(&state, state.eax, &filllen);
+		asm_movl(&state, state.ecx, &len);
+
+mixrPlayChannelplayecx:
+	asm_movl(&state, 0, &ramploop);
+	asm_movl(&state, 0, (uint32_t *)&ramping[0]);
+	asm_movl(&state, 0, (uint32_t *)&ramping[1]);
+
+	asm_cmpl(&state, 0, state.ecx);
+	asm_je(&state, mixrPlayChannelnoplay);
+
+	asm_movl(&state, chan->dstvols[0], &state.edx);
+	asm_subl(&state, chan->curvols[0], &state.edx);
+	asm_je(&state, mixrPlayChannelnoramp0);
+	asm_jl(&state, mixrPlayChannelramp0down);
+		asm_movl(&state, 1, (uint32_t *)&ramping[0]);
+		asm_cmpl(&state, state.edx, state.ecx);
+		asm_jbe(&state, mixrPlayChannelnoramp0);
+			asm_movl(&state, 1, &ramploop);
+			asm_movl(&state, state.edx, &state.ecx);
+			asm_jmp(&state, mixrPlayChannelnoramp0);
+mixrPlayChannelramp0down:
+	asm_negl(&state, &state.edx);
+	asm_movl(&state, -1, (uint32_t *)&ramping[0]);
+	asm_cmpl(&state, state.edx, state.ecx);
+	asm_jbe(&state, mixrPlayChannelnoramp0);
+		asm_movl(&state, 1, &ramploop);
+		asm_movl(&state, state.edx, &state.ecx);
+mixrPlayChannelnoramp0:
+
+	asm_movl(&state, chan->dstvols[1], &state.edx);
+	asm_subl(&state, chan->curvols[1], &state.edx);
+	asm_je(&state, mixrPlayChannelnoramp1);
+	asm_jl(&state, mixrPlayChannelramp1down);
+		asm_movl(&state, 1, (uint32_t *)&ramping[1]);
+		asm_cmpl(&state, state.edx, state.ecx);
+		asm_jbe(&state, mixrPlayChannelnoramp1);
+			asm_movl(&state, 1, &ramploop);
+			asm_movl(&state, state.edx, &state.ecx);
+			asm_jmp(&state, mixrPlayChannelnoramp1);
+mixrPlayChannelramp1down:
+		asm_negl(&state, &state.edx);
+		asm_movl(&state, -1, (uint32_t *)&ramping[1]);
+		asm_cmpl(&state, state.edx, state.ecx);
+		asm_jbe(&state, mixrPlayChannelnoramp1);
+			asm_movl(&state, 1, &ramploop);
+			asm_movl(&state, state.edx, &state.ecx);
+mixrPlayChannelnoramp1:
+
+	asm_movl(&state, /*(uint32_t)routeptr*/0x12345678, &state.edx); /* NC value */
+	asm_cmpl(&state, 0, ramping[0]);
+	asm_jne(&state, mixrPlayChannelnotquiet);
+	asm_cmpl(&state, 0, ramping[1]);
+	asm_jne(&state, mixrPlayChannelnotquiet);
+	asm_cmpl(&state, 0, chan->curvols[0]);
+	asm_jne(&state, mixrPlayChannelnotquiet);
+	asm_cmpl(&state, 0, chan->curvols[1]);
+	asm_jne(&state, mixrPlayChannelnotquiet);
+	asm_movl(&state, /*(uint32_t)routq*/0x12345678, &state.edx);
+		routeptr = routequiet; /* work-around */
+
+mixrPlayChannelnotquiet:
+	asm_movl(&state, 0x12345678+4, &state.ebx);
+	asm_movl(&state, chan->step, &state.eax);
+	asm_shll(&state, 16, &state.eax);
+	asm_movl(&state, state.eax, &step_high); /* ref ebx to reach step_high */
+	asm_movl(&state, 0x12345678+8, &state.ebx);
+	asm_movl(&state, chan->step, &state.eax);
+	asm_sarl(&state, 16, &state.eax);
+	asm_movl(&state, state.eax, &step_low); /* ref ebx to reach step_high */
+	asm_movl(&state, 0x12345678+12, &state.ebx);
+	asm_movl(&state, ramping[0], &state.eax);
+	asm_shll(&state, 8, &state.eax);
+	asm_movl(&state, state.eax, &ramping0); /* ref ebx to reach step_high */
+	asm_movl(&state, 0x12345678+16, &state.ebx);
+	asm_movl(&state, ramping[1], &state.eax);
+	asm_shll(&state, 8, &state.eax);
+	asm_movl(&state, state.eax, &ramping1); /* ref ebx to reach step_high */
+	asm_movl(&state, 0x12345678+20, &state.ebx);
+	bufferlen = state.ecx;
+	endofbuffer = buf[state.ecx];
+	asm_leal(&state, state.ecx*4, &state.eax);
+	asm_cmpl(&state, 0, stereo);
+	asm_je(&state, mixrPlayChannelm1);
+		asm_shll(&state, 1, &state.eax);
+mixrPlayChannelm1:
+	asm_addl(&state, 0x10000000, &state.eax); /* buf */
+	//asm_movl(&state, eax, *(uint32_t **)(state.ebx));  we save these above */
+
+	asm_pushl(&state, state.ecx);
+	/* asm_movl(&state, *(uint32_t *)(state.edx)), &state.eax); */ asm_movl(&state, 0x12345678, &state.eax);
+	
+	asm_movl(&state, chan->curvols[0], &state.ebx);
+	asm_shll(&state, 8, &state.ebx);
+	asm_movl(&state, chan->curvols[1], &state.ecx);
+	asm_shll(&state, 8, &state.ecx);
+	asm_movw(&state, chan->fpos, &state.dx);
+	asm_shll(&state, 16, &state.edx);
+	asm_movl(&state, chan->pos, &state.esi);
+	asm_addl(&state, /*chan->samp*/0x12345678, &state.esi);
+	asm_movl(&state, 0x10000000, &state.edi); /* buf */
+
+	/* asm_call(&state, *(uint32_t **)(state.eax)); */
+	if (buf<orgbuf)
+	{
+		fprintf(stderr, "#GP buf<orgbuf\n");
+	} else if ((buf+(len<<stereo)) > (orgbuf+(orglen<<stereo)))
+	{
+		fprintf(stderr, "#GP (buf+len) > (orgbuf+orglen)\n");
+	} else
+		routeptr(buf, bufferlen, chan);
+
+	asm_popl(&state, &state.ecx);
+	asm_movl(&state, 0xf0000000, &state.edi); /* &chan == 0xf0000000   only dummy, since we won't deref chan directly */
+
+mixrPlayChannelnoplay:
+	asm_movl(&state, state.ecx, &state.eax);
+	asm_shll(&state, 2, &state.eax);
+	asm_cmpl(&state, 0, stereo);
+	asm_je(&state, mixrPlayChannelm2);
+		asm_shll(&state, 1, &state.eax);
+mixrPlayChannelm2:
+	/*asm_addl(&state, state.eax, &buf);*/ buf = (int32_t *)((char *)buf + state.eax);
+	/*asm_subl(&state, state.ecx, &len);*/ len -= state.ecx;
+
+	asm_movl(&state, chan->step, &state.eax);
+	asm_imul_1_l(&state, state.ecx);
+	asm_shldl(&state, 16, state.eax, &state.edx);
+	asm_addw(&state, state.eax, &chan->fpos);
+	asm_adcl(&state, state.edx, &chan->pos);
+
+	asm_movl(&state, ramping[0], &state.eax);
+	asm_imul_2_l(&state, state.ecx, &state.eax);
+	asm_addl(&state, state.eax, (uint32_t *)chan->curvols+0);
+	asm_movl(&state, ramping[1], &state.eax);
+	asm_imul_2_l(&state, state.ecx, &state.eax);
+	asm_addl(&state, state.eax, (uint32_t *)chan->curvols+1);
+
+	asm_cmpl(&state, 0, ramploop);
+	asm_jnz(&state, mixrPlayChannelbigloop);
+
+	asm_cmpl(&state, 0, inloop);
+	asm_jz(&state, mixrPlayChannelfill);
+
+	asm_movl(&state, chan->pos, &state.eax);
+	asm_cmpl(&state, 0, chan->step);
+	asm_jge(&state, mixrPlayChannelforward2);
+		asm_cmpl(&state, chan->loopstart, state.eax);
+		asm_jge(&state, mixrPlayChannelexit);
+		asm_testb(&state, MIXRQ_PINGPONGLOOP, chan->status);
+
+		asm_jnz(&state, mixrPlayChannelpong);
+			asm_addl(&state, chan->replen, &state.eax);
+			asm_jmp(&state, mixrPlayChannelloopiflen);
+mixrPlayChannelpong:
+			asm_negl(&state, (uint32_t *)&chan->step);
+			asm_negw(&state, &chan->fpos);
+			asm_adcl(&state, 0, &state.eax);
+			asm_negl(&state, &state.eax);
+			asm_addl(&state, chan->loopstart, &state.eax);
+			asm_addl(&state, chan->loopstart, &state.eax);
+			asm_jmp(&state, mixrPlayChannelloopiflen);
+mixrPlayChannelforward2:
+		asm_cmpl(&state, chan->loopend, state.eax);
+		asm_jb(&state, mixrPlayChannelexit);
+		asm_testb(&state, MIXRQ_PINGPONGLOOP, chan->status);
+
+		asm_jnz(&state, mixrPlayChannelping);
+			asm_subl(&state, chan->replen, &state.eax);
+			asm_jmp(&state, mixrPlayChannelloopiflen);
+mixrPlayChannelping:
+			asm_negl(&state, (uint32_t *)&chan->step);
+			asm_negw(&state, &chan->fpos);
+			asm_adcl(&state, 0, &state.eax);
+			asm_negl(&state, &state.eax);
+			asm_addl(&state, chan->loopend, &state.eax);
+			asm_addl(&state, chan->loopend, &state.eax);
+mixrPlayChannelloopiflen:
+	asm_movl(&state, state.eax, &chan->pos);
+	asm_cmpl(&state, 0, len);
+	asm_jne(&state, mixrPlayChannelbigloop);
+	asm_jmp(&state, mixrPlayChannelexit);
+
+mixrPlayChannelfill:
+	asm_cmpl(&state, 0, filllen);
+	asm_je(&state, mixrPlayChannelfadechk);
+	asm_movl(&state, chan->length, &state.eax);
+	asm_movl(&state, state.eax, &chan->pos);
+	asm_addl(&state, /*chan->samp*/0x12345678, &state.eax);
+	asm_movl(&state, chan->curvols[0], &state.ebx);
+	asm_movl(&state, chan->curvols[1], &state.ecx);
+	asm_shll(&state, 8, &state.ebx);
+	asm_shll(&state, 8, &state.ecx);
+	asm_testb(&state, MIXRQ_PLAY16BIT, chan->status);
+
+	asm_jnz(&state, mixrPlayChannelfill16);
+		asm_movb(&state, chan->realsamp.bit8[state.eax-0x12345678], &state.bl);
+		asm_jmp(&state, mixrPlayChannelfilldo);
+mixrPlayChannelfill16:
+		asm_movb(&state, ((uint16_t *)chan->realsamp.bit16)[state.eax-0x12345678]>>8, &state.bl);
+mixrPlayChannelfilldo:
+	asm_movb(&state, state.bl, &state.cl);
+	asm_movl(&state, ((uint32_t *)mixrFadeChannelvoltab)[state.ebx], &state.ebx);
+	asm_movl(&state, ((uint32_t *)mixrFadeChannelvoltab)[state.ecx], &state.ecx);
+	asm_movl(&state, filllen, &state.eax);
+	asm_movl(&state, /*buf*/0x30000000, &state.edi);
+	asm_cmpl(&state, 0, stereo);
+	asm_jne(&state, mixrPlayChannelfillstereo);
+mixrPlayChannelfillmono:
+/*		asm_addl(&state, ebx, &(uint32_t *)state.edi);*/ buf[(state.edi-0x30000000)/4] += state.ebx;
+		asm_addl(&state, 4, &state.edi);
+	asm_decl(&state, &state.eax);
+	asm_jnz(&state, mixrPlayChannelfillmono);
+	asm_jmp(&state, mixrPlayChannelfade);
+mixrPlayChannelfillstereo:
+/*		asm_addl(&state, ebx, &(uint32_t *)state.edi);*/ buf[(state.edi-0x30000000)/4] += state.ebx;
+/*		asm_addl(&state, ecx, &(uint32_t *)(state.edi+4));*/ buf[(state.edi-0x30000000+4)/4] += state.ecx;
+		asm_addl(&state, 8, &state.edi);
+	asm_decl(&state, &state.eax);
+	asm_jnz(&state, mixrPlayChannelfillstereo);
+	asm_jmp(&state, mixrPlayChannelfade);
+
+mixrPlayChannelfadechk:
+	asm_cmpl(&state, 0, dofade);
+	asm_je(&state, mixrPlayChannelexit);
+mixrPlayChannelfade:
+	asm_movl(&state, /*chan*/0xf0000000, &state.edi);
+	asm_movl(&state, /*fadebuf*/0xe0000000, &state.esi);
+	/*asm_call(&state, mixrFadeChannel);*/ mixrFadeChannel(fadebuf, chan);
+mixrPlayChannelexit:
+	{
+	}
+}
+#else
+
 void mixrPlayChannel(int32_t *buf, int32_t *fadebuf, uint32_t len, struct channel *chan, int stereo)
 {
 	uint32_t fillen=0;
@@ -1327,7 +1712,7 @@ void mixrPlayChannel(int32_t *buf, int32_t *fadebuf, uint32_t len, struct channe
 	int dofade=0;
 	int route=0;
 	route_func routeptr;
-	uint32_t mylen;
+	uint32_t mylen; /* ecx */
 
 	stereo=!!stereo; /* make sure it is 0 or 1 */
 
@@ -1355,22 +1740,22 @@ mixrPlayChannelbigloop:
 
 		/* length = eax */
 		if (chan->step<0)
-		{
+		{ /* mixrPlayChannelbackward: */
 			mypos=chan->pos;
 			mystep=-chan->step;
 			myfpos=chan->fpos;
 			if (chan->status&MIXRQ_LOOPED)
 			{
-				if (chan->pos > chan->loopstart)
+				if (chan->pos >= chan->loopstart)
 				{
 					mypos -= chan->loopstart;
 					inloop = 1;
 				}
 			}
-		} else {
+		} else { /* mixrPlayChannelforward */
 			mystep=chan->step;
+			mypos=chan->length - chan->pos - (!chan->fpos);
 			myfpos=-chan->fpos;
-			mypos = chan->length - chan->pos - (!!chan->fpos);
 			if (chan->status&MIXRQ_LOOPED)
 			{
 				if (chan->pos < chan->loopend)
@@ -1381,18 +1766,23 @@ mixrPlayChannelbigloop:
 			}
 		}
 		mixrPlayChannelmaxplaylen: /* here we analyze how much we can sample */
-		{
+		{			
 			uint64_t tmppos;
-			tmppos=(((uint64_t)(mypos<<16))|myfpos)+((uint32_t)mystep)-1;
+/*			fprintf(stderr, "Samples available to use: %d/0x%08x\n", mypos, mypos);*/
+			tmppos=((((uint64_t)mypos)<<16)|myfpos)+((uint32_t)mystep)-1;
+/*			fprintf(stderr, "Samples available to use hidef: %lld/0x%012llx\n", tmppos, tmppos);*/
 			if ((tmppos>>32)<mystep)
 			{/* this is the safe check to avoid overflow in div */
 				uint32_t tmplen;
 				tmplen = tmppos / mystep; /* eax */
+/*				fprintf(stderr, "Samples at current playrate would be output into %d samples\n", tmplen);*/
 				if (mylen>=tmplen)
 				{
+/*					fprintf(stderr, "world wants more data than we can provide, limi output\n");*/
 					mylen=tmplen; /* ecx = eax */
 					if (!inloop)
 					{ /* add padding */
+/*						fprintf(stderr, "We are not in loop, quit sample\n");*/
 						dofade=1;
 						chan->status&=~MIXRQ_PLAYING;
 						fillen+=(len-mylen); /* the gap that is left */
@@ -1638,12 +2028,12 @@ mixrPlayChannelbigloop:
 		"  movw %15(%%edi), %%si\n"       /* %15 = ch->fpos */
 		"  movb $0, %9\n"                 /*  %9 = inloop */
 
-		edi=chan
 		eax=garbage/routeptr
-		ecx=len
 		ebx=ch->step
+		ecx=len
 		edx=ch->pos
-		si=ch->fpos
+		edi=chan
+		 si=ch->fpos
 		inloop=0
 
 		"  cmpl $0, %%ebx\n"
@@ -1660,7 +2050,7 @@ mixrPlayChannelbackward:
 		"    movl %%edx, %%eax\n"
 
 		eax=ch->pos
-		ebx=-ch->step
+		ebx=-ch->step (now positive)
 		ecx=len
 		edx=ch->pos
 		edi=chan
@@ -1673,7 +2063,7 @@ mixrPlayChannelbackward:
 		if (!chan->status&MIXRQ_LOOPED)
 			goto mixrPlayChannelmaxplaylen
 
-		"    cmpl %16(%%edi), %%edx\n"    /* %16 = ch->loopstart */
+		"    cmpi %16(%%edi), %%edx\n"    /* %16 = ch->loopstart */
 		"    jb mixrPlayChannelmaxplaylen\n"
 
 		if (ch->pos < ch->loopstart)
@@ -1698,7 +2088,7 @@ mixrPlayChannelforward:
 		"    negw %%si\n"
 		"    sbbl %%edx, %%eax\n"
 
-		eax = chan->length - ch->pos - (!!ch->fpos)
+		eax = chan->length - ch->pos - (!ch->fpos before it was negated)
 		ebx = ch->step
 		ecx = len
 		edx = ch->pos
@@ -1726,6 +2116,15 @@ mixrPlayChannelmaxplaylen:
 
 		"  xorl %%edx, %%edx\n"
 		eax = length stuff            (length)
+		/* if we play reverse with no loop, we can play upto pos samples */
+                      ch->pos                  (reverse play)
+		/* if we play reverse with loop, but no loop reached, we can play upto pos samples */
+                      ch->pos - ch->loopstart  (reverse play, if pos was after loopstart forward motion )
+		/* if we play forward with no loop, we can play upto length - pos samples */
+		      ch->length - ch->pos - CF (forward play)
+		/* if we play forward with loop, and we are inside the loop, we can play until the loopend...   loopend - pos*/
+	              ch->length - ch->pos - CF - ch->length + chan->loopend  (forward play, if pos was after loop end)
+		      ch->loopend - ch->pos - CF (simplyfied above)
 		ebx = abs(ch->step)           (mystep)
 		ecx = len
 		edx = 0
@@ -2198,6 +2597,7 @@ mixrPlayChannelfade:
 	);
 	#endif
 }
+#endif
 
 void mixrFade(int32_t *buf, int32_t *fade, int len, int stereo)
 {
