@@ -1,7 +1,7 @@
 #define MIXF_MIXBUFLEN 4096
 #define MIXF_MAXCHAN 255
 
-#define MIXF_PLAYSTEREO 1
+//#define MIXF_PLAYSTEREO 0     Floating point mixer doesn't support stereo samples, only mono to stereo output
 #define MIXF_INTERPOLATE 2
 #define MIXF_INTERPOLATEQ 4
 #define MIXF_FILTER 8
@@ -17,45 +17,6 @@
 #define MIXF_VOLRAMP  256
 #define MIXF_DECLICK  512
 
-struct channel
-{
-	void *samp;
-
-	uint32_t length;
-	uint32_t loopstart;
-	uint32_t loopend;
-
-	int   newsamp;
-
-	float dstvols[2];
-	int   dontramp;
-
-	float vol[2];
-	float orgvol[2];
-
-	float orgvolx;
-	float orgpan;
-	float orgfrez;
-
-	float *sbpos;
-	float sbuf[8];
-
-	uint32_t orgrate;
-	uint32_t orgfrq;
-	uint32_t orgdiv;
-	int      volopt;
-	uint32_t samptype;
-
-	uint32_t orgloopstart;
-	uint32_t orgloopend;
-	uint32_t orgsloopstart;
-	uint32_t orgsloopend;
-
-	long  handle;
-};
-
-
-
 extern void mixer (void);
 /*#pragma aux mixer "*" parm[] modify [eax ebx ecx edx esi edi ebp];*/
 
