@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * assembler routines for Quality Mixer
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -93,10 +107,10 @@ void noexternal_dwmixq(void)
 		"  movb 0(%esi), %cl\n"
 		"  movb 1(%esi), %al\n"
 		"  addl %edx, %ebx\n"
-		"  movw %cs:1234(,%ecx,4), %bx\n"
+		"  movw %es:1234(,%ecx,4), %bx\n"
 		"    playmonoi_intr1:\n"
 		"  adcl %ebp, %esi\n"
-		"  addw %cs:1234(, %eax, 4), %bx\n"
+		"  addw %es:1234(, %eax, 4), %bx\n"
 		"    playmonoi_intr2:\n"
 
 		"    movw %bx, -2(%edi)\n"
@@ -128,13 +142,13 @@ void noexternal_dwmixq(void)
 		"    movl %ecx, %eax\n"
 		"    movb 1(%esi,%esi), %cl\n"
 		"    movb 3(%esi,%esi), %al\n"
-		"    movw %cs:1234(,%ecx,4), %bx\n"
+		"    movw %es:1234(,%ecx,4), %bx\n"
 		"      playmonoi16_intr1:\n"
 		"    movb (%esi, %esi), %cl\n"
 		"    addw 1234(,%eax,4), %bx\n"
 		"      playmonoi16_intr2:\n"
 		"    movb 2(%esi, %esi), %al\n"
-		"    addw %cs:1234(,%ecx,4), %bx\n"
+		"    addw %es:1234(,%ecx,4), %bx\n"
 		"      playmonoi16_intr3:\n"
 		"    addl %edx, %ebx\n"
 		"    adcl %ebp, %esi\n"
@@ -175,7 +189,7 @@ void noexternal_dwmixq(void)
 		"    movb 0(%esi), %cl\n"
 		"    movb 1(%esi), %al\n"
 
-		"    movw %cs:1234(,%ecx,8), %bx\n"
+		"    movw %es:1234(,%ecx,8), %bx\n"
 		"      playmonoi2_intr1:\n"
 		"    movb 2(%esi), %cl\n"
 		"    addw 1234(,%eax,8), %bx\n"
@@ -183,7 +197,7 @@ void noexternal_dwmixq(void)
 
 		"    addl %edx, %ebx\n"
 		"    adcl %ebp, %esi\n"
-		"    addw %cs:1234(,%ecx,8), %bx\n"
+		"    addw %es:1234(,%ecx,8), %bx\n"
 		"      playmonoi2_intr3:\n"
 
 		"    movw %bx, -2(%edi)\n"
@@ -220,7 +234,7 @@ void noexternal_dwmixq(void)
 		"    movb 1(%esi, %esi), %cl\n"
 		"    movb 3(%esi, %esi), %al\n"
 
-		"    movw %cs:1234(,%ecx,8), %bx\n"
+		"    movw %es:1234(,%ecx,8), %bx\n"
 		"      playmonoi216_intr1:\n"
 
 		"    movb 5(%esi, %esi), %cl\n"
@@ -228,7 +242,7 @@ void noexternal_dwmixq(void)
 		"      playmonoi216_intr2:\n"
 
 		"    movb 0(%esi,%esi), %al\n"
-		"    addw %cs:1234(,%ecx,8), %bx\n"
+		"    addw %es:1234(,%ecx,8), %bx\n"
 		"      playmonoi216_intr3:\n"
 
 		"    movb 2(%esi,%esi), %cl\n"
@@ -236,7 +250,7 @@ void noexternal_dwmixq(void)
 		"      playmonoi216_intr4:\n"
 
 		"    movb 4(%esi, %esi), %al\n"
-		"    addw %cs:1234(,%ecx,8), %bx\n"
+		"    addw %es:1234(,%ecx,8), %bx\n"
 		"      playmonoi216_intr5:\n"
 
 		"    addl %edx, %ebx\n"
@@ -789,7 +803,7 @@ static void playmono(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step>>16;
+	uint32_t posadd=(int16_t)(chan->step>>16);
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -809,7 +823,7 @@ static void playmono16(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step>>16;
+	uint32_t posadd=(int16_t)(chan->step>>16);
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -829,7 +843,7 @@ static void playmonoi(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step>>16;
+	uint32_t posadd=(int16_t)(chan->step>>16);
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -852,7 +866,7 @@ static void playmonoi16(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step>>16;
+	uint32_t posadd=(int16_t)(chan->step>>16);
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -877,11 +891,12 @@ static void playmonoi16(int16_t *buf, uint32_t len, struct channel *chan)
 	}
 }
 
+
 static void playmonoi2(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int16_t  posadd=chan->step>>16;
+	uint32_t posadd=(int16_t)(chan->step>>16);
 	uint32_t pos=chan->pos;
 
 	while (len)
@@ -901,6 +916,7 @@ static void playmonoi2(int16_t *buf, uint32_t len, struct channel *chan)
 		}
 		pos+=posadd;
 		len--;
+
 	}
 }
 
@@ -908,7 +924,7 @@ static void playmonoi216(int16_t *buf, uint32_t len, struct channel *chan)
 {
 	uint32_t fpos=chan->fpos;
 	uint32_t fadd=chan->step&0xffff;
-	int32_t  posadd=chan->step>>16;
+	uint32_t posadd=(int16_t)(chan->step>>16);
 	uint32_t pos=chan->pos;
 	while (len)
 	{
@@ -994,21 +1010,21 @@ mixqPlayChannel_bigloop:
 			mystep=-chan->step;
 			myfpos=chan->fpos;
 			if (chan->status&MIXQ_LOOPED)
-			{
-				if (chan->pos>=chan->loopstart)
-				{
-					mypos -= chan->loopstart;
+			{				
+				if (chan->pos >= chan->loopstart)
+				{					
+					mypos -= chan->loopstart;		
 					inloop = 1;
 				}
 			}
 		} else {
 		mixqPlayChannel_forward:
 			mystep=chan->step;
+			mypos=chan->length - chan->pos - (!chan->fpos);
 			myfpos=-chan->fpos;
-			mypos = chan->length - chan->pos - (!!myfpos);
 			if (chan->status&MIXQ_LOOPED)
 			{
-				if (chan->pos<chan->loopend)
+				if (chan->pos < chan->loopend)
 				{
 					mypos -=  chan->length - chan->loopend;
 					inloop = 1;
@@ -1018,19 +1034,30 @@ mixqPlayChannel_bigloop:
 		/* mypos should now be the end of the loop envelope */
 	mixqPlayChannel_maxplaylen:
 		{
-			uint64_t tmppos;/* signed to avoid some warnings */
-			tmppos=(((uint64_t)(mypos<<16))|myfpos)+((uint32_t)mystep)-1;
+			uint64_t tmppos;
+/*			fprintf(stderr, "Samples available to use: %d/0x%08x\n", mypos, mypos); */
+			tmppos=((((uint64_t)mypos)<<16)|myfpos)+((uint32_t)mystep)-1;
+/*			fprintf(stderr, "Samples available to use hidef: %lld/0x%012llx\n", tmppos, tmppos);*/
 			if ((tmppos>>32)<mystep)
 			{
+				/* mypos = samples until barrier */
+				/* tmppos = mypos.myfpos + mystep-1 = 32.16 sample pos after route? */
 				uint32_t tmplen;
 				tmplen=tmppos/mystep;
+/*				fprintf(stderr, "Samples at current playrate would be output into %d samples\n", tmplen); */
+				/* tmplen = samples of output data can this generate */
 				if (mylen>=tmplen)
 				{
+					/* output data requested is more than can be generated */
+/*					fprintf(stderr, "world wants more data than we can provide, limit output\n");*/
 					mylen=tmplen;
+					/* data requested is now equal to what can be generated */
 					if (!inloop)
 					{
+/*						fprintf(stderr, "We are not in loop, quit sample\n");*/
+						/* no loop.. we hit EOF */
 						chan->status&=~MIXQ_PLAYING;
-						fillen+=(tmplen-mylen);
+						fillen+=(len-tmplen);
 						len=mylen;
 					}
 				}
@@ -1053,7 +1080,7 @@ mixqPlayChannel_playecx:
 
 	if (inloop)
 	{
-		uint32_t mypos = chan->pos; /* eax */
+		int32_t mypos = chan->pos; /* eax */
 		if (chan->step<0)
 		{
 			if (mypos>=(int32_t)chan->loopstart)
@@ -1198,7 +1225,7 @@ mixqPlayChannel_bigloop:
 			goto mixqPlayChannel_maxplaylen;
 
 		"    cmpl %11(%%edi), %%edx\n"    /* %11 = ch->loopstart */
-		"    jb mixqPlayChannel_maxplaylen\n"
+		"    jl mixqPlayChannel_maxplaylen\n"
 		if (chan->pos<ch->loopstart)
 			goto mixqPlayChannel_maxplaylen;
 		"    subl %11(%%edi), %%eax\n"    /* %11 = ch->loopstart */
