@@ -1,7 +1,21 @@
 /* OpenCP Module Player
- * copyright (c) '04-'09 Stian Skjelstad <stian@nixia.no>
+ * copyright (c) '04-'10 Stian Skjelstad <stian@nixia.no>
  *
  * Unit-test for "dwmixa.c"
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "config.h"
@@ -14,12 +28,10 @@
 #ifdef I386_ASM
 #include <unistd.h>
 #include <sys/mman.h>
-#include <limits.h>    /* for PAGESIZE */
-#ifndef PAGESIZE
-#define PAGESIZE 4096
-#endif
+#include "stuff/pagesize.inc.c"
 #endif
 
+
 static int16_t (*amptab)[256]; /* signedness is not fixed here */
 static int signedout;
 static uint32_t clipmax;
@@ -257,10 +269,10 @@ static int initAsm(void)
 		/*fprintf(stderr, "range2: %p - %p\n", start2, stop2);*/
 #endif
 
-		start1=(char *)(((int)start1)&~(PAGESIZE-1));
-		/*start2=(char *)(((int)start2)&~(PAGESIZE-1));*/
-		len1=((stop1-start1)+PAGESIZE-1)& ~(PAGESIZE-1);
-		/*len2=((stop2-start2)+PAGESIZE-1)& ~(PAGESIZE-1);*/
+		start1=(char *)(((int)start1)&~(pagesize()-1));
+		/*start2=(char *)(((int)start2)&~(pagesize()-1));*/
+		len1=((stop1-start1)+pagesize()-1)& ~(pagesize()-1);
+		/*len2=((stop2-start2)+pagesize-1)& ~(pagesize()-1);*/
 #ifdef MIXER_DEBUG
 		fprintf(stderr, "mprot: %p + %08x\n", start1, len1);
 		/*fprintf(stderr, "mprot: %p + %08x\n", start2, len2);*/
@@ -293,8 +305,8 @@ static int initAsm(void)
 			return 0;
 		}
 		*/
-/*	if (mprotect((char *)(((int)remap_range1_start)&~(PAGESIZE-1)), (((char *)remap_range1_stop-(char *)remap_range1_start)+PAGESIZE-1)& ~(PAGESIZE-1), PROT_READ|PROT_WRITE|PROT_EXEC) ||
-	    mprotect((char *)(((int)remap_range2_start)&~(PAGESIZE-1)), (((char *)remap_range2_stop-(char *)remap_range2_start)+PAGESIZE-1)& ~(PAGESIZE-1), PROT_READ|PROT_WRITE|PROT_EXEC) )
+/*	if (mprotect((char *)(((int)remap_range1_start)&~(pagesize()-1)), (((char *)remap_range1_stop-(char *)remap_range1_start)+pagesize()-1)& ~(pagesize()-1), PROT_READ|PROT_WRITE|PROT_EXEC) ||
+	    mprotect((char *)(((int)remap_range2_start)&~(pagesize()-1)), (((char *)remap_range2_stop-(char *)remap_range2_start)+pagesize()-1)& ~(pagesize()-1), PROT_READ|PROT_WRITE|PROT_EXEC) )
 	    {
 			perror("Couldn't mprotect"); 
  			return 0;
