@@ -1,15 +1,25 @@
 /* OpenCP Module Player
- * copyright (c) '04-'09 Stian Skjelstad <stian@nixia.no>
+ * copyright (c) '04-'10 Stian Skjelstad <stian@nixia.no>
  *
  * Unit-test for "dwmixqa.c"
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */     
 
 #include "config.h"
 #include <sys/mman.h>
-#include <limits.h>    /* for PAGESIZE */
-#ifndef PAGESIZE
-#define PAGESIZE 4096
-#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -17,8 +27,11 @@
 #include "dev/mix.h"
 #include "devwmix.h"
 #include "dwmix.h"
-/*#include "dwmixa.h"*/
 #include "dwmixqa.h"
+#ifdef I386_ASM
+#include "stuff/pagesize.inc.c"
+#endif
+
 
 static int initAsm(void)
 {
@@ -34,8 +47,8 @@ static int initAsm(void)
 	start1=(void *)remap_range2_start;
 	stop1=(void *)remap_range2_stop;
 
-	start1=(char *)(((int)start1)&~(PAGESIZE-1));
-	len1=((stop1-start1)+PAGESIZE-1)& ~(PAGESIZE-1);
+	start1=(char *)(((int)start1)&~(pagesize()-1));
+	len1=((stop1-start1)+pagesize()-1)& ~(pagesize()-1);
 	if (write(fd, start1, len1)!=len1)
 	{
 		close(fd);
@@ -58,8 +71,8 @@ static int initAsm(void)
 }
 
 static int16_t (*voltabsq)[2][256];
-static uint16_t (*interpoltabq)[32][256][2];
-static uint16_t (*interpoltabq2)[16][256][4];
+static int16_t (*interpoltabq)[32][256][2];
+static int16_t (*interpoltabq2)[16][256][4];
 
 static void calcvoltabsq(void)
 {
