@@ -1,7 +1,21 @@
 /* OpenCP Module Player
- * copyright (c) '94-'09 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * database for storing a tree of filenames
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */ 
 
 #include "config.h"
@@ -20,10 +34,12 @@ struct dirdbEntry
 {
 	uint32_t parent;
 	uint32_t mdbref;
+	uint32_t adbref;
 	char *name; /* we pollute malloc a lot with this */
 	int refcount;
 
-	uint32_t newmdbref;
+	uint32_t newadbref;
+	uint32_t newmdbref; /* used during scan to find new nodes */
 };
 static uint32_t tagparentnode = DIRDB_NOPARENT;
 
@@ -33,6 +49,8 @@ struct __attribute__((packed)) dirdbheader
 	uint32_t entries;
 };
 const char dirdbsigv1[60] = "Cubic Player Directory Data Base\x1B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";
+const char dirdbsigv2[60] = "Cubic Player Directory Data Base\x1B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01";
+
 
 
 static struct dirdbEntry *dirdbData=0;
@@ -51,7 +69,7 @@ static void dumpdb_parent(uint32_t parent, int ident)
 			fprintf(stderr, "0x%08x ", i);
 			for (j=0;j<ident;j++)
 				fprintf(stderr, " ");
-			fprintf(stderr, "%s (mdb 0x%08x)\n", dirdbData[i].name, dirdbData[i].mdbref);
+			fprintf(stderr, "%s (mdb 0x%08x adb 0x%08x)\n", dirdbData[i].name, dirdbData[i].mdbref, dirdbData[i].adbref);
 			dumpdb_parent(i, ident+1);
 		}
 	}
@@ -70,6 +88,7 @@ int dirdbInit(void)
 	int f;
 	uint32_t i;
 	int retval;
+	int version;
 
 	if ((strlen(cfConfigDir)+11)>PATH_MAX)
 	{
@@ -95,10 +114,15 @@ int dirdbInit(void)
 	}
 	if (memcmp(header.sig, dirdbsigv1, 60))
 	{
-		fprintf(stderr, "Invalid header\n");
-		close(f);
-		return 1;
-	}
+		if (memcmp(header.sig, dirdbsigv2, 60))
+		{
+			fprintf(stderr, "Invalid header\n");
+			close(f);
+			return 1;
+		} else
+			version = 2;
+	} else
+		version = 1;
 	dirdbNum=uint32_little(header.entries);
 	if (!dirdbNum)
 		goto endoffile;
@@ -130,6 +154,14 @@ int dirdbInit(void)
 				goto endoffile;
 			dirdbData[i].mdbref = uint32_little(dirdbData[i].mdbref);
 
+			if (version == 2)
+			{
+				if (read(f, &dirdbData[i].adbref, sizeof(uint32_t))!=sizeof(uint32_t))
+					goto endoffile;
+				dirdbData[i].adbref = uint32_little(dirdbData[i].adbref);
+			} else
+				dirdbData[i].adbref = DIRDB_NO_ADBREF;
+
 			dirdbData[i].name=malloc(len+1);
 			if (!dirdbData[i].name)
 				goto outofmemory;
@@ -231,6 +263,7 @@ reentry:
 			dirdbData[i].parent=parent;
 			dirdbData[i].refcount++;
 			dirdbData[i].mdbref=DIRDB_NO_MDBREF;
+			dirdbData[i].adbref=DIRDB_NO_ADBREF;
 			if (parent!=DIRDB_NOPARENT)
 			{
 				/*fprintf(stderr, "  + %s (%d p=%d)\n", dirdbData[i].name, i, dirdbData[i].parent);*/
@@ -256,6 +289,8 @@ reentry:
 		uint32_t j;
 		for (j=i;j<dirdbNum;j++)
 		{
+			dirdbData[j].adbref=DIRDB_NO_ADBREF;
+			dirdbData[j].newadbref=DIRDB_NO_ADBREF;
 			dirdbData[j].mdbref=DIRDB_NO_MDBREF;
 			dirdbData[j].newmdbref=DIRDB_NO_MDBREF;
 		}
@@ -390,8 +425,11 @@ err:
 	free(dirdbData[node].name);
 	dirdbData[node].name=0;
 
-	dirdbData[node].mdbref=0xffffffff; /* this should not be needed */
-	dirdbData[node].newmdbref=0xffffffff; /* this should not be needed */
+	dirdbData[node].mdbref=DIRDB_NO_MDBREF; /* this should not be needed */
+	dirdbData[node].newmdbref=DIRDB_NO_MDBREF; /* this should not be needed */
+	dirdbData[node].adbref=DIRDB_NO_ADBREF; /* this should not be needed */
+	dirdbData[node].newadbref=DIRDB_NO_ADBREF; /* this should not be needed */
+
 #ifdef DIRDB_DEBUG
 	dumpdirdb();
 #endif
@@ -495,7 +533,7 @@ void dirdbFlush(void)
 		if (dirdbData[i].name)
 			max=i+1;
 
-	memcpy(header.sig, dirdbsigv1, sizeof(dirdbsigv1));
+	memcpy(header.sig, dirdbsigv2, sizeof(dirdbsigv2));
 	header.entries=uint32_little(max);
 
 	if (write(f, &header, sizeof(header))!=sizeof(header))
@@ -513,6 +551,9 @@ void dirdbFlush(void)
 			if (write(f, &buf32, sizeof(uint32_t))!=sizeof(uint32_t))
 				goto writeerror;
 			buf32=uint32_little(dirdbData[i].mdbref);
+			if (write(f, &buf32, sizeof(uint32_t))!=sizeof(uint32_t))
+				goto writeerror;
+			buf32=uint32_little(dirdbData[i].adbref);
 			if (write(f, &buf32, sizeof(uint32_t))!=sizeof(uint32_t))
 				goto writeerror;
 			if (dirdbData[i].name)
@@ -583,7 +624,10 @@ void dirdbTagSetParent(uint32_t node)
 	}
 
 	for (i=0;i<dirdbNum;i++)
+	{
 		dirdbData[i].newmdbref=DIRDB_NO_MDBREF;
+		dirdbData[i].newadbref=DIRDB_NO_ADBREF; /* is this actually needed? */
+	}
 
 	if (node>=dirdbNum)
 	{
@@ -594,13 +638,14 @@ void dirdbTagSetParent(uint32_t node)
 	dirdbRef(node);
 }
 
-void dirdbMakeMdbRef(uint32_t node, uint32_t mdbref)
+void dirdbMakeMdbAdbRef(uint32_t node, uint32_t mdbref, uint32_t adbref)
 {
 	if (node>=dirdbNum)
 	{
 		fprintf(stderr, "dirdbMakeMdbRef: invalid node\n");
 		return;
 	}
+	/* the madness below is in order to keep track of references the correct way */
 	if (mdbref==DIRDB_NO_MDBREF)
 	{
 		if (dirdbData[node].newmdbref!=DIRDB_NO_MDBREF)
@@ -614,9 +659,13 @@ void dirdbMakeMdbRef(uint32_t node, uint32_t mdbref)
 			dirdbData[node].newmdbref=mdbref;
 			dirdbRef(node);
 		} else {
+			/*dirdbUnref(node);*/
 			dirdbData[node].newmdbref=mdbref;
+			/*dirdbRef(node);  overkill to unref and re-ref just for the name's sake*/
 		}
 	}
+
+	dirdbData[node].newadbref = adbref;
 }
 
 void dirdbTagCancel(void)
@@ -629,6 +678,7 @@ void dirdbTagCancel(void)
 			dirdbData[i].newmdbref=DIRDB_NO_MDBREF;
 			dirdbUnref(i);
 		}
+		dirdbData[i].newadbref = DIRDB_NO_ADBREF;
 	}
 	if (tagparentnode==DIRDB_NOPARENT)
 	{
@@ -639,13 +689,14 @@ void dirdbTagCancel(void)
 	tagparentnode=DIRDB_NOPARENT;
 }
 
-void _dirdbTagRemoveUntaggedAndSubmit(uint32_t node)
+static void _dirdbTagRemoveUntaggedAndSubmit(uint32_t node)
 {
 	uint32_t i;
 	for (i=0;i<dirdbNum;i++)
 	{
 		if (dirdbData[i].parent==node)
 		{
+			dirdbData[i].adbref=dirdbData[i].newadbref;
 			if (dirdbData[i].newmdbref==dirdbData[i].mdbref)
 			{
 				if (dirdbData[i].mdbref==DIRDB_NO_MDBREF)
@@ -663,18 +714,21 @@ void _dirdbTagRemoveUntaggedAndSubmit(uint32_t node)
 				{
 					dirdbData[i].mdbref=dirdbData[i].newmdbref;
 					dirdbData[i].newmdbref=DIRDB_NO_MDBREF;
-					/* no need to unref/ref, since we are balanced.
-					 * If somebody has named a file, the same name
-					 * a directory used to have, we will be a
-					 * bit f*cked here, but the cost to scan the
-					 * database here is to expensive by my meenings.
-					 * NB NB NB NB
+					/* no need to unref/ref, since we are
+					 * balanced. Since somebody can have
+					 * named a file, the same name
+					 * a directory used to have, we need
+					 * to scan for siblings.
 					 */
+					_dirdbTagRemoveUntaggedAndSubmit(i);
+					/* this can probably be done more elegant later */
 				} else if (dirdbData[i].newmdbref==DIRDB_NO_MDBREF)
 				{
 					dirdbData[i].mdbref=DIRDB_NO_MDBREF;
 					dirdbUnref(i);
 					/* same as above regarding renaming */
+					_dirdbTagRemoveUntaggedAndSubmit(i);
+					/* this can probably be done more elegant later */
 				} else {
 					dirdbData[i].mdbref=dirdbData[i].newmdbref;
 					dirdbData[i].newmdbref=DIRDB_NO_MDBREF;
@@ -699,11 +753,12 @@ void dirdbTagRemoveUntaggedAndSubmit(void)
 	dirdbDirty=1;
 }
 
-int dirdbGetMdb(uint32_t *dirdbnode, uint32_t *mdbnode, int *first)
+int dirdbGetMdbAdb(uint32_t *dirdbnode, uint32_t *mdbnode, uint32_t *adbref, int *first)
 {
 	if (*first)
 	{
 		*dirdbnode=0;
+		*adbref=DIRDB_NO_ADBREF;
 		*first=0;
 	} else
 		(*dirdbnode)++;
@@ -711,8 +766,8 @@ int dirdbGetMdb(uint32_t *dirdbnode, uint32_t *mdbnode, int *first)
 		if ((dirdbData[*dirdbnode].name)&&(dirdbData[*dirdbnode].mdbref!=DIRDB_NO_MDBREF))
 		{
 			*mdbnode=dirdbData[*dirdbnode].mdbref;
+			*adbref=dirdbData[*dirdbnode].adbref;
 			return 0;
 		}
 	return -1;
 }
-
