@@ -9,7 +9,8 @@ extern int fsGetPrevFile(char *path, struct moduleinfostruct *info, FILE **); /*
 extern int fsFilesLeft(void);
 extern signed int fsFileSelect(void);
 /* extern char fsAddFiles(const char *);      use the playlist instead..*/
-extern char fsInit(void);
+extern int fsPreInit(void);
+extern int fsInit(void);
 extern void fsClose(void);
 
 extern int fsFPS; /* see stuff/framelock.c */
