@@ -1,14 +1,29 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * *.M3u file-reader/parser
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -ss051231  Stian Skjelstad <stian@nixia.no>
  *    -first release
  */
 
 #include "config.h"
+#include <ctype.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <fnmatch.h>
@@ -43,6 +58,11 @@ static int m3uReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	int fd;
 	struct stat st;
 
+#ifndef FNM_CASEFOLD
+	char *mask_upper;
+	char *iterate;
+#endif
+
 	if (drive!=dmFILE)
 		return 1;
 	
@@ -51,7 +71,9 @@ static int m3uReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	/* Does the file end in .M3U ? */
 	s1=newpath+strlen(newpath)-4;
 	if (s1<newpath)
+	{
 		return 1;
+	}
 	if (strcasecmp(s1, ".M3U"))
 		return 1;
 	
@@ -91,6 +113,17 @@ static int m3uReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	buftail=readbuffer;
 	buftail_n=st.st_size;
 
+#ifndef FNM_CASEFOLD
+	if ((mask_upper = strdup(mask)))
+	{
+		for (iterate = mask_upper; *iterate; iterate++)
+			*iterate = toupper(*iterate);
+	} else {
+		perror("pfsm3u.c: strdup() failed");
+		return 1;
+	}
+#endif
+
 	while (buftail_n>0)
 	{
 		/* find new-line */
@@ -113,103 +146,19 @@ static int m3uReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 			goto newline;
 		if (!buftail[0])
 			goto newline;
-#if 1
-		fsAddPlaylist(ml, newpath, mask, opt, buftail);
+#ifndef FNM_CASEFOLD
+		fsAddPlaylist(ml, newpath, mask_upper, opt, buftail);
 #else
-		{
-			const char *dmDrive=NULL;
-			char fullpath[PATH_MAX+1];
-			struct stat st;
-			struct modlistentry retval;
-
-			if (s2[0]!='/')
-			{
-				if ((s3=index(s2, '/')))
-					if (s3[-1]==':')
-					{
-						if (!(dmDrive=dmFindDrive(s2)))
-						{
-							*s3=0;
-							fprintf(stderr, "[M3U] Drive/Protocol not supported (%s)\n", s2);
-							/* Drive/Protocol not supported */
-							goto newline;
-						}
-						s2+=strlen(dmDrive);
-					}
-			}
-			if (!dmDrive)
-				dmDrive=drive;
-
-			gendir(newpath, s2, fullpath);
-			if ((s3=rindex(fullpath, '/')))
-				s3++;
-			else
-				s3=fullpath;
-
-			if (strcmp(dmDrive, "file:"))
-			{
-				fprintf(stderr, "[M3U], API for getting handlers via dmDrive needed. TODO\n");
-				goto newline;
-			}
-			
-			memset(&st, 0, sizeof(st));
-			memset(&retval, 0, sizeof(retval));
-
-			if (stat(fullpath, &st)<0)
-			{
-				fprintf(stderr, "[M3U] stat() failed for %s\n", fullpath);
-				goto newline;
-			}
-
-			retval.drive=dmDrive;
-			strncpy(retval.name, s3, NAME_MAX);
-			retval.name[NAME_MAX]=0;
-
-			strncpy(retval.fullname, fullpath, PATH_MAX);
-			retval.fullname[PATH_MAX]=0;
-
-			fs12name(retval.shortname, s3);	
-
-			if (S_ISREG(st.st_mode))
-			{
-				if (isarchivepath(retval.fullname))
-				{
-					retval.flags=MODLIST_FLAG_ARC;
-					strncat(retval.fullname, "/", PATH_MAX-strlen(retval.fullname)-1);
-				} else {
-					char curext[NAME_MAX+1];
-					_splitpath(retval.fullname, 0, 0, 0, curext);
-				      	if ((fnmatch(mask, retval.name, FNM_CASEFOLD))||(!fsIsModule(curext)))
-						goto newline;
-					retval.fileref=mdbGetModuleReference(retval.shortname, st.st_size);
-					retval.flags=MODLIST_FLAG_FILE;
-				}
-			} else if (S_ISDIR(st.st_mode))
-			{
-				if ((opt&RD_PUTDSUBS))
-				{
-					retval.flags=MODLIST_FLAG_DIR;
-					strncat(retval.fullname, "/", PATH_MAX-strlen(retval.fullname)-1);
-				} else if ((opt&RD_PUTRSUBS))
-				{
-					strncat(retval.fullname, "/", PATH_MAX-strlen(retval.fullname)-1);
-					fsReadDir(ml,drive, retval.fullname, mask, opt);
-					goto newline;
-				} else
-					goto newline;
-			} else
-				goto newline;
-			retval.Read=dosfile_Read;
-			retval.ReadHeader=dosfile_ReadHeader;
-			retval.ReadHandle=dosfile_ReadHandle;
-			modlist_append(ml, &retval); /* this call no longer can fail */
-		}
+		fsAddPlaylist(ml, newpath, mask, opt, buftail);
 #endif
+
 newline:
 		buftail_n-=(s1-buftail)+1;
 		buftail=s1+1;
 	}
-
+#ifndef FNM_CASEFOLD
+	free(mask_upper);
+#endif
 	free(readbuffer);
 	return 1;
 }
