@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * Main routine, calls fileselector and interface code
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -351,7 +365,7 @@ static int _fsMain(int argc, char *argv[])
 	return errOk;
 }
 
-static int fsint(void)
+static int fspreint(void)
 {
 	mdbRegisterReadDir(&adbReadDirReg);
 	mdbRegisterReadDir(&dosReadDirReg);
@@ -361,6 +375,17 @@ static int fsint(void)
 	mdbRegisterReadInfo(&fsReadInfoReg);
 	
 	fprintf(stderr, "initializing fileselector...\n");
+	if (!fsPreInit())
+	{
+		fprintf(stderr, "fileselector pre-init failed!\n");
+		return errGen;
+	}
+
+	return errOk;
+}
+
+static int fsint(void)
+{
 	if (!fsInit())
 	{
 		fprintf(stderr, "fileselector init failed!\n");
@@ -370,7 +395,6 @@ static int fsint(void)
 	return errOk;
 }
 
-
 static void fsclose()
 {
 	mdbUnregisterReadDir(&adbReadDirReg);
@@ -590,4 +614,4 @@ static void __attribute__((destructor))done(void)
 char *dllinfo = "";
 #endif
 
-DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"pfilesel", "OpenCP Fileselector (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0, PreInit: fsint, LateClose: fsclose};
+DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {"pfilesel", "OpenCP Fileselector (c) 1994-10 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", DLLVERSION, 0, PreInit: fspreint, Init: fsint, LateClose: fsclose};
