@@ -1,14 +1,29 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * *.PLS file-reader/parser
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -ss051231  Stian Skjelstad <stian@nixia.no>
  *    -first release
  */
 
 #include "config.h"
+#include <ctype.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <fnmatch.h>
@@ -42,6 +57,11 @@ static int plsReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	
 	int fd;
 	struct stat st;
+
+#ifndef FNM_CASEFOLD
+	char *mask_upper;
+	char *iterate;
+#endif
 	
 	if (drive!=dmFILE)  /* we, only support file:// transport for now... TODO */
 		return 1;
@@ -91,6 +111,17 @@ static int plsReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 	buftail=readbuffer;
 	buftail_n=st.st_size;
 
+#ifndef FNM_CASEFOLD
+	if ((mask_upper = strdup(mask)))
+	{
+		for (iterate = mask_upper; *iterate; iterate++)
+			*iterate = toupper(*iterate);
+	} else {
+		perror("pfspls.c: strdup() failed");
+		return 1;
+	}
+#endif
+
 	while (buftail_n>0)
 	{
 		/* find new-line */
@@ -114,12 +145,18 @@ static int plsReadDir(struct modlist *ml, const struct dmDrive *drive, const uin
 		/* skip the =, and check that the line has a length */
 		if (!*(++s2))
 			goto newline;
+#ifndef FNM_CASEFOLD
+		fsAddPlaylist(ml, newpath, mask_upper, opt, s2);
+#else
 		fsAddPlaylist(ml, newpath, mask, opt, s2);
+#endif
 newline:
 		buftail_n-=(s1-buftail)+1;
 		buftail=s1+1;
 	}
-
+#ifndef FNM_CASEFOLD
+	free(mask_upper);
+#endif
 	free(readbuffer);
 	return 1;
 }
