@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * MEDIALIBRARY filebrowser
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -ss050430   Stian Skjelstad <stian@nixia.no>
  *    -first release
@@ -96,7 +110,7 @@ static int mlSubScan(const uint32_t dirdbnode, int mlTop)
 		{
 			if (!mdbInfoRead(mle->fileref))
 				mdbScan(mle);
-			dirdbMakeMdbRef(mle->dirdbfullpath, mle->fileref);
+			dirdbMakeMdbAdbRef(mle->dirdbfullpath, mle->fileref, mle->adb_ref);
 		}
 	}
 	modlist_free(ml);
@@ -318,7 +332,8 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 		int first=1;
 		uint32_t dirdbnode;
 		uint32_t mdbref;
-		while (!dirdbGetMdb(&dirdbnode, &mdbref, &first))
+		uint32_t adbref;
+		while (!dirdbGetMdbAdb(&dirdbnode, &mdbref, &adbref, &first))
 		{
 			char cachefile[NAME_MAX+1];
 			dirdbGetname(dirdbnode, cachefile);
@@ -330,10 +345,17 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 			entry.dirdbfullpath=dirdbnode; /*dirdbResolvePathAndRef(files[i].name);*/
 			entry.flags=MODLIST_FLAG_FILE;
 			entry.fileref=mdbref;
-			entry.adb_ref=0xffffffff;
-			entry.Read=dosfile_Read;
-			entry.ReadHeader=dosfile_ReadHeader;
-			entry.ReadHandle=dosfile_ReadHandle;
+			entry.adb_ref=adbref;
+			if (adbref==DIRDB_NO_ADBREF)
+			{
+				entry.Read=dosfile_Read;
+				entry.ReadHeader=dosfile_ReadHeader;
+				entry.ReadHandle=dosfile_ReadHandle;
+			} else {
+				entry.Read=adb_Read;
+				entry.ReadHeader=adb_ReadHeader;
+				entry.ReadHandle=adb_ReadHandle;
+			}
 			modlist_append(ml, &entry);
 /*			dirdbUnref(entry.dirdbfullpath);*/
 		}
@@ -433,13 +455,14 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 						int first=1;
 						uint32_t dirdbnode;
 						uint32_t mdbref;
+						uint32_t adbref;
 
 						setcurshape(0);
 
 						for (i=0;i<cmdlen;i++)
 							str[i]=toupper(str[i]);
 						
-						while (!dirdbGetMdb(&dirdbnode, &mdbref, &first))
+						while (!dirdbGetMdbAdb(&dirdbnode, &mdbref, &adbref, &first))
 						{
 							char cachefile[NAME_MAX+1];
 							struct moduleinfostruct info;
@@ -488,10 +511,17 @@ static int mlReadDir(struct modlist *ml, const struct dmDrive *drive, const uint
 								entry.dirdbfullpath=dirdbnode; /*dirdbResolvePathAndRef(files[i].name);*/
 								entry.flags=MODLIST_FLAG_FILE;
 								entry.fileref=mdbref;
-								entry.adb_ref=0xffffffff;
-								entry.Read=dosfile_Read;
-								entry.ReadHeader=dosfile_ReadHeader;
-								entry.ReadHandle=dosfile_ReadHandle;
+								entry.adb_ref=adbref;
+								if (adbref==DIRDB_NO_ADBREF)
+								{
+									entry.Read=dosfile_Read;
+									entry.ReadHeader=dosfile_ReadHeader;
+									entry.ReadHandle=dosfile_ReadHandle;
+								} else {
+									entry.Read=adb_Read;
+									entry.ReadHeader=adb_ReadHeader;
+									entry.ReadHandle=adb_ReadHandle;
+								}
 								modlist_append(ml, &entry);
 					/*			dirdbUnref(entry.dirdbfullpath);*/
 							}
@@ -529,4 +559,4 @@ static void mlclose(void)
 static struct mdbreaddirregstruct mlReadDirReg = {mlReadDir MDBREADDIRREGSTRUCT_TAIL};
 
 char *dllinfo = "";
-struct linkinfostruct dllextinfo = {"medialib", "OpenCP medialib (c) 2005 Stian Skjelstad", DLLVERSION, 0, Init: mlint, Close: mlclose, PreInit:0, LateInit: 0, PreClose:0, LateClose: 0};
+struct linkinfostruct dllextinfo = {"medialib", "OpenCP medialib (c) 2005-09 Stian Skjelstad", DLLVERSION, 0, Init: mlint, Close: mlclose, PreInit:0, LateInit: 0, PreClose:0, LateClose: 0};
