@@ -1,5 +1,5 @@
 /* aylet 0.3, a .AY music file player.
- * Copyright (C) 2001-2002 Russell Marks and Ian Collier.
+ * Copyright (C) 2001-2010 Russell Marks and Ian Collier.
  * See main.c for licence.
  *
  * ayplay.c - the sound emulation itself, based on the beeper/AY code I
@@ -659,9 +659,9 @@ void __attribute__ ((visibility ("internal"))) ayIdle(void)
 
 
 				c0 = *(uint16_t *)(aybuf+(aybufpos<<2))^0x8000;
-				vm1= *(uint16_t *)(aybuf+aym1)^0x8000;
-				v1 = *(uint16_t *)(aybuf+ay1)^0x8000;
-				v2 = *(uint16_t *)(aybuf+ay2)^0x8000;
+				vm1= *(uint16_t *)(aybuf+(aym1<<2))^0x8000;
+				v1 = *(uint16_t *)(aybuf+(ay1<<2))^0x8000;
+				v2 = *(uint16_t *)(aybuf+(ay2<<2))^0x8000;
 				c1 = v1-vm1;
 				c2 = 2*vm1-2*c0+v1-v2;
 				c3 = c0-vm1-v1+v2;
@@ -677,9 +677,9 @@ void __attribute__ ((visibility ("internal"))) ayIdle(void)
 					ls=65535;
 
 				c0 = *(uint16_t *)(aybuf+(aybufpos<<2)+2)^0x8000;
-				vm1= *(uint16_t *)(aybuf+aym1+2)^0x8000;
-				v1 = *(uint16_t *)(aybuf+ay1+2)^0x8000;
-				v2 = *(uint16_t *)(aybuf+ay2+2)^0x8000;
+				vm1= *(uint16_t *)(aybuf+(aym1<<2)+2)^0x8000;
+				v1 = *(uint16_t *)(aybuf+(ay1<<2)+2)^0x8000;
+				v2 = *(uint16_t *)(aybuf+(ay2<<2)+2)^0x8000;
 				c1 = v1-vm1;
 				c2 = 2*vm1-2*c0+v1-v2;
 				c3 = c0-vm1-v1+v2;
