@@ -1,5 +1,5 @@
 /* aylet 0.3, a .AY music file player.
- * Copyright (C) 2001-2002 Russell Marks and Ian Collier.
+ * Copyright (C) 2001-2010 Russell Marks and Ian Collier.
  * See main.c for licence.
  *
  * sound.c - the sound emulation itself, based on the beeper/AY code I
@@ -103,7 +103,8 @@ static struct ay_change_tag ay_change[AY_CHANGE_MAX];
 static int ay_change_count;
 
 static int fading=0,fadetotal;
-static int fadetime;
+static int sfadetime;
+
 
 #define STEREO_BUF_SIZE 1024
 
@@ -567,7 +568,7 @@ for(f=0;f<16;f++)
 for(f=0;f<3;f++)
   ay_tone_high[f]=0;
 ay_tone_subcycles=ay_env_subcycles=0;
-fading=fadetime=0;
+fading=sfadetime=0;
 sound_oldval=sound_oldval_orig=0;
 
 CLOCK_RESET(AY_CLOCK);	/* in case it was CPC before */
@@ -650,7 +651,7 @@ silent_level=sound_buf[fulllen-1];
 /* apply overall fade if we're in the middle of one. */
 if(fading)
   {
-  if(fadetime<=0)
+  if(sfadetime<=0)
     memset(sound_buf,0,fulllen*sizeof(int16_t)),silent=1;
   else
     {
@@ -658,12 +659,12 @@ if(fading)
     for(f=0;f<sound_framesiz;f++,ptr++)
       {
       /* XXX kludgey, but needed to avoid overflow */
-      fadetime--;
-      *ptr=(*ptr)*(fadetime>>4)/(fadetotal>>4);
+      sfadetime--;
+      *ptr=(*ptr)*(sfadetime>>4)/(fadetotal>>4);
       if(sound_stereo)
         {
         ptr++;
-	*ptr=(*ptr)*(fadetime>>4)/(fadetotal>>4);
+	*ptr=(*ptr)*(sfadetime>>4)/(fadetotal>>4);
         }
       }
     }
@@ -707,7 +708,7 @@ ay_driver_frame(buf,fulllen);
 void __attribute__ ((visibility ("internal"))) sound_start_fade(int fadetime_in_sec)
 {
 fading=1;
-fadetime=fadetotal=fadetime_in_sec*sound_freq;
+sfadetime=fadetotal=fadetime_in_sec*sound_freq;
 }
 
 void __attribute__ ((visibility ("internal"))) sound_beeper(int on)
