@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * GMDPlay loader for PolyTracker modules
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -92,6 +106,8 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 
 	if (fread(patpara, 256, 1, file) != 1)
 		fprintf(stderr, __FILE__ ": warning, read failed #3\n");
+	for (i=0;i<128;i++)
+		patpara[i] = uint16_little(patpara[i]);
 
 	if (!mpAllocInstruments(m, m->instnum)||!mpAllocTracks(m, m->tracknum)||!mpAllocPatterns(m, m->patnum)||!mpAllocSamples(m, m->sampnum)||!mpAllocModSamples(m, m->modsampnum)||!mpAllocOrders(m, m->ordnum))
 		return errAllocMem;
@@ -634,4 +650,4 @@ static int _mpLoadPTM(struct gmdmodule *m, FILE *file)
 
 struct gmdloadstruct mpLoadPTM = { _mpLoadPTM };
 
-struct linkinfostruct dllextinfo = {"gmdlptm", "OpenCP Module Loader: *.PTM (c) 1994-04 Niklas Beisert", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+struct linkinfostruct dllextinfo = {"gmdlptm", "OpenCP Module Loader: *.PTM (c) 1994-10 Niklas Beisert, Stian Skjelstad", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
