@@ -1,8 +1,23 @@
 /* OpenCP Module Player
- * copyright (c) '94-'98 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * GMIPlay MID/RMI file loader
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -88,7 +103,7 @@ int __attribute__ ((visibility ("internal")))
 
 __attribute__ ((visibility ("internal"))) char midInstrumentNames[256][NAME_MAX+1];
 
-static inline unsigned long readvlnum(unsigned char **ptr)
+static inline unsigned long readvlnum(unsigned char **ptr, unsigned char *endptr, int *eof)
 {
 	unsigned long num=0;
 	while (1)
@@ -96,6 +111,11 @@ static inline unsigned long readvlnum(unsigned char **ptr)
 #ifdef MIDI_LOAD_DEBUG
 		fprintf(stderr, "0x%02x ", **ptr);
 #endif
+		if(*ptr>=endptr)
+		{
+			*eof=1;
+			break; /* END OF DATA */
+		}
 		num=(num<<7)|((**ptr)&0x7F);
 		if (!(*((*ptr)++)&0x80))
 			break;
@@ -314,13 +334,19 @@ char __attribute__ ((visibility ("internal"))) midLoadMidi( struct midifile *m,
 		uint8_t *trkptr=m->tracks[i].trk;
 		uint8_t status=0;
 		uint32_t trackticks=0;
+		int eof=0;
 		while (trkptr<m->tracks[i].trkend)
 		{
 #ifdef MIDI_LOAD_DEBUG
 			fprintf(stderr, "[midi-load] Adding new event\n    ");
 #endif
-#warning TODO, no ptrlen checking here
-			trackticks+=readvlnum(&trkptr);
+			trackticks+=readvlnum(&trkptr, m->tracks[i].trkend, &eof);
+			if (eof)
+			{
+				m->ticknum=0; /* abort the song */
+				fprintf(stderr, "[midi-load] #1 premature end-of-data in track %d\n", i);
+				break;
+			}
 #ifdef MIDI_LOAD_DEBUG
 			fprintf(stderr, " timeslice\n");
 			//fprintf(stderr, "[midi-load] New trackticks is now: %d\n", (int)trackticks);
@@ -330,7 +356,15 @@ char __attribute__ ((visibility ("internal"))) midLoadMidi( struct midifile *m,
 				fprintf(stderr, "           event type is cached\n");
 #endif
 			if (*trkptr&0x80)
+			{
 				status=*trkptr++;
+				if (trkptr>m->tracks[i].trkend)
+				{
+					m->ticknum=0; /* abort the song */
+					fprintf(stderr, "[midi-load] #2 premature end-of-data in track %d\n", i);
+					break;
+				}
+			}
 			else if ((status&0xf0) == 0xf0)
 			{
 				fprintf(stderr, "[midi-load] cached status 0xFn is not supposed to happen\n");
@@ -344,12 +378,33 @@ char __attribute__ ((visibility ("internal"))) midLoadMidi( struct midifile *m,
 					fprintf(stderr, "    0x%02x META EVENT type\n", status);
 #endif
 					trkptr++;
+					if (trkptr>=m->tracks[i].trkend)
+					{
+						m->ticknum=0; /* abort the song */
+						fprintf(stderr, "[midi-load] #3 premature end-of-data in track %d\n", i);
+						break;
+					}				
 				}
 #ifdef MIDI_LOAD_DEBUG
 				fprintf(stderr, "    ");
 #endif
-				len=readvlnum(&trkptr);
+				len=readvlnum(&trkptr, m->tracks[i].trkend, &eof);
+ 				if (eof)
+				{
+					m->ticknum=0; /* abort the song */
+					fprintf(stderr, "[midi-load] #4 premature end-of-data in track %d\n", i);
+					break;
+				}
+
 				trkptr+=len;
+
+				if (trkptr>m->tracks[i].trkend)
+				{
+					m->ticknum=0; /* abort the song */
+					fprintf(stderr, "[midi-load] #5 premature end-of-data in track %d\n", i);
+					break;
+				}
+
 #ifdef MIDI_LOAD_DEBUG
 				fprintf(stderr, " length.. skipping it\n");
 #endif
@@ -357,6 +412,12 @@ char __attribute__ ((visibility ("internal"))) midLoadMidi( struct midifile *m,
 				switch (status&0xF0)
 				{
 					case 0x90: /* note on */
+						if ((trkptr+1)>=m->tracks[i].trkend)
+						{
+							m->ticknum=0; /* abort the song */
+							fprintf(stderr, "[midi-load] #6 premature end-of-data in track %d\n", i);
+							break;
+						}								
 						if (trkptr[1])
 						{
 							sampused[chaninst[status&0xF]][trkptr[0]>>3]|=1<<(trkptr[0]&7);
@@ -368,6 +429,12 @@ char __attribute__ ((visibility ("internal"))) midLoadMidi( struct midifile *m,
 					case 0xA0: /* aftertouch */
 					case 0xB0: /* control-change */
 					case 0xE0: /* pitch wheel */
+						if ((trkptr+1)>=m->tracks[i].trkend)
+						{
+							m->ticknum=0; /* abort the song */
+							fprintf(stderr, "[midi-load] #7 premature end-of-data in track %d\n", i);
+							break;
+						}								
 						trkptr+=2;
 						break;
 					case 0xD0: /* Channel pressure */
