@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'98 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * GMIPlay - SMF file player using GUS patches
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -186,6 +200,12 @@ static void playnote(uint8_t chan,
 	int i;
 	int16_t noteval;
 
+	if (note>=sizeof(instr[mc->ins].note))
+	{
+		fprintf(stderr, "[gmiplay] #1 note out of range: %d\n", note);
+		return;
+	}
+
 	if (instr[mc->ins].note[note]==0xFF)
 		return;
 
@@ -264,7 +284,13 @@ static void noteoff(uint8_t chan, uint8_t note)
 {
 	struct mchandata *mc=&mchan[chan];
 	int i;
-  
+ 
+	if (note>=sizeof(instr[mc->ins].note))
+	{
+		fprintf(stderr, "[gmiplay] #2 note out of range: %d\n", note);
+		return;
+	}
+
 	for (i=0; i<MAXCHANNOTE; i++)
 		if ((mc->note[i]==note)&&pchan[mc->pch[i]].sus)
 			break;
@@ -327,6 +353,7 @@ static void playticks(struct mtrackdata *t, uint32_t ticks)
 			t->tickpos+=ticks;
 			break;
 		}
+		/* these three readvlnum should be safe, since gmiload.c accepted them */
 		ticks-=readvlnum(&t->pos.trk)-t->tickpos;
 		t->tickpos=0;
 		if (*t->pos.trk&0x80)
