@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * ITPlay .IT module loader
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -156,7 +170,7 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 	}
 	
 	signedsamp=!(hdr.cwtv<0x202);
-	this->deltapacked=(hdr.cmwt==0x215);
+	this->deltapacked=(hdr.cmwt>=0x215);
 
 	if (fread(ords, hdr.nords*sizeof(uint8_t), 1, file) != 1)
 	{
@@ -563,6 +577,12 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 			continue;
 		}
 
+		if (!(shdr.length))
+		{
+			fprintf(stderr, "itload.c: A sample with length=0, skipped\n");
+			continue;
+		}
+
 		sp->vol=shdr.vol;
 		sp->gvl=shdr.gvl;
 		sp->vir=shdr.vir;
@@ -583,6 +603,8 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 		sip->type=((shdr.flags&2)?mcpSamp16Bit:0)|((shdr.flags&4)?mcpSampStereo:0)|(signedsamp?0:mcpSampUnsigned)|((shdr.flags&0x10)?mcpSampLoop:0)|((shdr.flags&0x40)?mcpSampBiDi:0)|((shdr.flags&0x80)?mcpSampSBiDi:0)|((shdr.flags&0x20)?mcpSampSLoop:0);
 
 		sp->packed=(shdr.flags&8?1:0);
+		if (sp->packed && this->deltapacked && (shdr.cvt & 4))
+			sp->packed|=2;
 
 		if (!(sip->ptr=malloc((sip->length+512)<<((sip->type&mcpSamp16Bit)?1:0))))
 		{
@@ -602,13 +624,13 @@ int __attribute__ ((visibility ("internal"))) it_load(struct it_module *this, FI
 		
 		if (sp->packed) {
 			if (sip->type & mcpSamp16Bit)
-				decompress16(file, sip->ptr, sip->length, this->deltapacked);
+				decompress16(file, sip->ptr, sip->length, sp->packed&2);
 			else
-				decompress8(file, sip->ptr, sip->length, this->deltapacked);
+				decompress8(file, sip->ptr, sip->length, sp->packed&2);
 		} else {
 			if (fread(sip->ptr, sip->length<<((sip->type&mcpSamp16Bit)?1:0), 1, file) != 1)
 			{
-				fprintf(stderr, "[IT]: fread() failed #14\n");
+				fprintf(stderr, "[IT]: fread() failed #14 (sip-ptr=%p sip->length=%d 16bit=%d)\n", sip->ptr, (int)sip->length, !!(sip->type&mcpSamp16Bit));
 				return errFileRead;
 			}
 		}
