@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * ITPlayer - player for Impulse Tracker 2.xx modules
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -74,7 +88,7 @@
 #define random(this) it_random(this)
 static int it_random(struct itplayer *this)
 {
-       	this->randseed=this->randseed*0x15A4E35+1;
+       	this->randseed=this->randseed*0x15A4E35+12345;
 	return (this->randseed>>16)&32767;
 }
 
@@ -130,7 +144,14 @@ static void playnote(struct itplayer *this, struct it_logchan *c, const uint8_t
 	const struct it_instrument *in;
 
 	if (cmd[0])
+	{
 		c->lastnote=cmd[0];
+		/* this is kind of wierd, but we need to know of retrigger changes already now */
+		if (cmd[3]==cmdRetrigger)
+			c->retrigpos=cmd[4]>>4|!(cmd[4]&0xf0); /* |!(cmd[4]&0xf0) makes value 0, into 1 */
+		else
+			c->retrigpos=c->retrigspd;
+	}
 
 	if (cmd[1])
 	{
@@ -157,7 +178,6 @@ static void playnote(struct itplayer *this, struct it_logchan *c, const uint8_t
 			c->pch->notefade=1;
 		return;
 	}
-
 	if ((c->lastins>this->ninst)||!c->lastins||!c->lastnote||(c->lastnote>=cmdNNoteFade))
 		return;
 
@@ -165,12 +185,43 @@ static void playnote(struct itplayer *this, struct it_logchan *c, const uint8_t
 
 	porta=(cmd[3]==cmdPortaNote)||(cmd[3]==cmdPortaVol)||((cmd[2]>=cmdVPortaNote)&&(cmd[2]<(cmdVPortaNote+10)));
 
-	if (!c->pch || c->pch->dead || !c->pch->fadeval)
+	if (!c->pch || c->pch->dead || !c->pch->fadeval) 
 		porta=0;
 
 	if (instchange || !porta)
 		c->fnotehit=1;
 
+	do
+	{
+		if (!((!cmd[0]) && cmd[1] && (!instchange) && (!porta) && (cmd[3]!=cmdOffset) && c->pch))
+			break;
+
+/* http://eval.sovietrussia.org/wiki/Player_abuse_tests#Ping-pong_loop_and_sample_number
+ *
+ * Bug-fix workaround for cases where you just set instrument number with no note, should not re-trigger note,
+ * just reset volumes
+ */
+		if (!(p=c->pch))
+			break;
+		if (p->notecut)
+			break;
+		if (p->dead)
+			break;
+		if (!(s=p->smp))
+			break;
+		if (!(in=p->inst))
+			break;
+		c->nna=in->nna;
+		c->vol=((100-in->rv)*s->vol+in->rv*(rand()%65))/100;
+		c->fvol=c->vol;
+		c->pan=(in->dfp&128)?c->cpan:in->dfp;
+		c->pan=(s->dfp&128)?(s->dfp&127):c->pan;
+		c->pan=range64(c->pan+(((c->lastnote-cmdNNote-in->ppc)*in->pps)>>8));
+		c->pan=/*(in->dfp&128)?c->pan:*/range64(c->pan+((in->rp*(rand() %129 -64))>>6));
+		c->fpan=c->pan;
+		return;
+	} while(0); /* runs atleast one time */
+
 	if (c->fnotehit)
 	{
 		int smp;
@@ -205,7 +256,7 @@ static void playnote(struct itplayer *this, struct it_logchan *c, const uint8_t
 			c->pch=&c->newchan;
 			c->pch->avibpos=0;
 			c->pch->avibdep=0;
-			c->retrigpos=0;
+			/* c->retrigpos=0; */
 		}
 		if (this->samples[smp-1].handle==0xFFFF)
 		{
@@ -309,7 +360,7 @@ static void playnote(struct itplayer *this, struct it_logchan *c, const uint8_t
 			c->pan=(in->dfp&128)?c->cpan:in->dfp;
 			c->pan=(s->dfp&128)?(s->dfp&127):c->pan;
 			c->pan=range64(c->pan+(((c->lastnote-cmdNNote-in->ppc)*in->pps)>>8));
-			c->pan=(in->dfp&128)?c->pan:range64(c->pan+((in->rp*(rand() %129 -64))>>6));
+			c->pan=/*(in->dfp&128)?c->pan:*/range64(c->pan+((in->rp*(rand() %129 -64))>>6));
 			c->fpan=c->pan;
 			c->cutoff=(in->ifp&128)?in->ifp:c->cutoff;
 			c->fcutoff=c->cutoff;
@@ -365,20 +416,29 @@ static void playvcmd(struct itplayer *this, struct it_logchan *c, int vcmd)
 	{
       		if (vcmd!=cmdVPortaD)
 			c->porta=4*(vcmd-cmdVPortaD);
+		c->vporta=c->porta;
 		c->fpitchslide=ifxPSDown;
 	} else if ((vcmd>=cmdVPortaU)&&(vcmd<(cmdVPortaU+10)))
 	{
 		if (vcmd!=cmdVPortaU)
 			c->porta=4*(vcmd-cmdVPortaU);
+		c->vporta=c->porta;
 		c->fpitchslide=ifxPSUp;
 	} else if ((vcmd>=cmdVPortaNote)&&(vcmd<(cmdVPortaNote+10)))
 	{
 		if (vcmd!=cmdVPortaNote)
 		{
-			c->portanote="\x00\x01\x04\x08\x10\x20\x40\x60\x80\xFF"[vcmd-cmdVPortaNote];
-			if (!this->geffect)
-				c->porta=c->portanote;
+			int tmp="\x00\x01\x04\x08\x10\x20\x40\x60\x80\xFF"[vcmd-cmdVPortaNote];
+			if (this->geffect)
+				c->portanote=tmp;
+			else
+				c->porta=tmp;
 		}
+		if (this->geffect)
+			c->vportanote=c->portanote;
+		else
+			c->vporta=c->porta;
+
 	   	c->fpitchslide=ifxPSToNote;
 	} else if ((vcmd>=cmdVVibrato)&&(vcmd<(cmdVVibrato+10)))
 	{
@@ -444,10 +504,10 @@ static void doretrigger(struct it_logchan *c)
 	int x;
 	struct it_physchan *p;
 
-	c->retrigpos++;
-	if (c->retrigpos<=c->retrigspd)
+	c->retrigpos--;
+	if (c->retrigpos)
 		return;
-	c->retrigpos=0;
+	c->retrigpos=c->retrigspd;
 	x=c->vol;
 	switch (c->retrigvol)
 	{
@@ -468,11 +528,19 @@ static void doretrigger(struct it_logchan *c)
 
 static void dotremor(struct it_logchan *c)
 {
-	if (c->tremorpos>=c->tremoroff)
-		c->fvol=0;
-	c->tremorpos++;
-	if (c->tremorpos>=c->tremorlen)
-		c->tremorpos=0;
+	if (c->tremoroncounter)
+		c->tremoroncounter--;
+	if (!c->tremoroncounter)
+	{
+		if (c->tremoroffcounter)
+		{
+			c->fvol=0;
+			c->tremoroffcounter--;
+		} else {
+			c->tremoroncounter=c->tremoron;
+			c->tremoroffcounter=c->tremoroff;
+		}
+	}
 }
 
 static int ishex(uint8_t c)
@@ -562,24 +630,31 @@ static void playcmd(struct itplayer *this, struct it_logchan *c, int cmd, int da
 		case cmdPortaD:
 			if (data)
 				c->porta=data;
+			c->eporta=c->porta;
 			c->fpitchslide=ifxPSDown;
 			c->fx=ifxPitchSlideDown;
-			c->fpitch=c->pitch=c->pitch+4*rowudslide(c->porta);
+			c->fpitch=c->pitch=c->pitch+4*rowudslide(c->eporta);
 			break;
 		case cmdPortaU:
 			if (data)
 				c->porta=data;
+			c->eporta=c->porta;
 			c->fpitchslide=ifxPSUp;
 			c->fx=ifxPitchSlideUp;
-			c->fpitch=c->pitch=c->pitch-4*rowudslide(c->porta);
+			c->fpitch=c->pitch=c->pitch-4*rowudslide(c->eporta);
 			break;
 		case cmdPortaNote:
 			if (data)
 			{
-				c->portanote=data;
-				if (!this->geffect)
-					c->porta=c->portanote;
+				if (this->geffect)
+					c->portanote=data;
+				else
+					c->porta=data;
 			}
+			if (this->geffect)
+				c->eportanote=c->portanote;
+			else
+				c->eporta=c->porta;
 			c->fpitchslide=ifxPSToNote;
 			c->fx=ifxPitchSlideToNote;
 			break;
@@ -593,10 +668,15 @@ static void playcmd(struct itplayer *this, struct it_logchan *c, int cmd, int da
 			dovibrato(this, c);
 			break;
 		case cmdTremor:
-			if (data&0xF0)
-				c->tremoroff=(data>>4)+(this->oldfx);
-			if (data&0xF)
-				c->tremorlen=c->tremoroff+(data&0xF)+(this->oldfx);
+			if (data)
+			{
+				c->tremoron=(data>>4)+(this->oldfx);
+				c->tremoroff=(data&0xF)+(this->oldfx);
+			}
+			if (!c->tremoroff)
+				c->tremoroff=1;
+			if (!c->tremoron)
+				c->tremoron=1;
 			c->fvolfx=ifxVXTremor;
 			c->fx=ifxTremor;
 			dotremor(c);
@@ -690,9 +770,11 @@ static void playcmd(struct itplayer *this, struct it_logchan *c, int cmd, int da
 		case cmdRetrigger:
 			if (data)
 			{
-				c->retrigspd=data&0xF;
+				c->retrigspd=data&0xF;				
 				c->retrigvol=data>>4;
 			}
+			if (!c->retrigspd)
+				c->retrigspd=1;
 			c->fx=ifxRetrig;
 			doretrigger(c);
 			break;
@@ -814,14 +896,17 @@ static void playcmd(struct itplayer *this, struct it_logchan *c, int cmd, int da
 					{
 						if (!c->specialdata)
 							c->patloopstart=this->currow;
-						else {
-							c->patloopcount++;
-							if (c->patloopcount<=c->specialdata)
+						else
+						{
+							if (!c->patloopcount)
+								c->patloopcount=c->specialdata;
+							else
+								c->patloopcount--;
+							if (c->patloopcount)
 							{
 								this->gotorow=c->patloopstart;
 								this->gotoord=this->curord;
 							} else {
-								c->patloopcount=0;
 								c->patloopstart=this->currow+1;
 							}
 						}
@@ -829,13 +914,18 @@ static void playcmd(struct itplayer *this, struct it_logchan *c, int cmd, int da
 					break;
 				case cmdSNoteCut:
 					c->fx=ifxNoteCut;
+					if (!c->specialdata)
+						c->specialdata++; /* SC0 should do the same as SC1 */
 					break;
 				case cmdSNoteDelay:
 					c->fx=ifxDelay;
+					if (!c->specialdata)
+						c->specialdata++; /* SD0 should do the same as SD1 */
 					dodelay(this, c);
 					break;
 				case cmdSPatDelayRow:
-					this->patdelayrow=c->specialdata;
+					if (!this->patdelayrow) /* only use the first command, per row */
+						this->patdelayrow=c->specialdata+1;
 					break;
 				case cmdSSetMIDIMacro:
 					c->sfznum=c->specialdata;
@@ -943,43 +1033,65 @@ static void dopanbrello(struct itplayer *this, struct it_logchan *c)
 			c->panbrrnd=random(this);
 		}
 		c->fpan=range64(c->fpan+((c->panbrdep*((c->panbrrnd&255)-128))>>6));
-		return;
+	} else {
+		switch (c->panbrtype)
+		{
+			case 0: x=sintab[c->panbrpos&255]*2; break;
+			case 1: x=128-(c->panbrpos&255); break;
+			default: /* remove a warning */
+			case 2: x=128-2*(c->panbrpos&128); break;
+		}
+		c->fpan=range64(c->fpan+((c->panbrdep*x)>>6));
 	}
-	switch (c->panbrtype)
-	{
-		case 0: x=sintab[c->panbrpos&255]*2; break;
-		case 1: x=128-(c->panbrpos&255); break;
-		default: /* remove a warning */
-		case 2: x=128-2*(c->panbrpos&128); break;
-	}
-	c->fpan=range64(c->fpan+((c->panbrdep*x)>>6));
 	c->panbrpos+=c->panbrspd;
 }
 
-static void doportanote(struct itplayer *this, struct it_logchan *c)
+static void doportanote(struct itplayer *this, struct it_logchan *c, int v) /* if v, then do vporta, else eporta */
 {
+
+	if (!c->dpitch)
+		return;
+
 	if (c->pitch<c->dpitch)
 	{
-		c->pitch+=(this->geffect?c->portanote:c->porta)*16;
+		if (v)
+			c->pitch+=(this->geffect?c->vportanote:c->vporta)*16;
+		else
+			c->pitch+=(this->geffect?c->eportanote:c->eporta)*16;
+	
 		if (c->pitch>c->dpitch)
 			c->pitch=c->dpitch;
 	} else {
-		c->pitch-=(this->geffect?c->portanote:c->porta)*16;
+		if (v)
+			c->pitch-=(this->geffect?c->vportanote:c->vporta)*16;
+		else
+			c->pitch-=(this->geffect?c->eportanote:c->eporta)*16;	
 		if (c->pitch<c->dpitch)
 			c->pitch=c->dpitch;
 	}
 	c->fpitch=c->pitch;
+	if (c->pitch==c->dpitch)
+		c->dpitch=0;
 }
 
 static void dodelay(struct itplayer *this, struct it_logchan *c)
 {
-       	if (this->curtick==c->specialdata)
+       	if ((this->curtick==c->specialdata))
 	{
 		if (c->delayed[0]||c->delayed[1])
 			playnote(this, c, c->delayed);
 		if (c->delayed[2])
 			playvcmd(this, c, c->delayed[2]);
+	} else if (((this->curtick+1)==(this->speed+this->patdelaytick))&&(!this->patdelayrow))
+	{
+/* Impulse Tracker has a bug where a too long delay affects the lastinstrument, even if no note were played.
+ *
+ * http://eval.sovietrussia.org/wiki/Player_abuse_tests#Out-of-range_note_delays
+ */
+		if (c->delayed[1])
+			c->lastins=c->delayed[1];
 	}
+
 }
 
 static int rangepitch(struct itplayer *this, int p)
@@ -997,22 +1109,6 @@ static const uint16_t arpnotetab[] =
 
 static void processfx(struct itplayer *this, struct it_logchan *c)
 {
-	if ((c->vcmd>=cmdVVolSlD)&&(c->vcmd<(cmdVVolSlD+10)))
-		c->fvol=c->vol=range64(c->vol-c->vvolslide);
-	else if ((c->vcmd>=cmdVVolSlU)&&(c->vcmd<(cmdVVolSlU+10)))
-		c->fvol=c->vol=range64(c->vol+c->vvolslide);
-	else if ((c->vcmd>=cmdVVibrato)&&(c->vcmd<(cmdVVibrato+10)))
-		dovibrato(this, c);
-	else if ((c->vcmd>=cmdVPortaNote)&&(c->vcmd<(cmdVPortaNote+10)))
-		doportanote(this, c);
-	else if ((c->vcmd>=cmdVPortaU)&&(c->vcmd<(cmdVPortaU+10)))
-	{
-		c->fpitch=c->pitch=rangepitch(this, c->pitch-c->porta*16);
-		if ((c->pitch==this->pitchhigh)&&c->pch)
-			c->pch->notecut=1;
-	} else if ((c->vcmd>=cmdVPortaD)&&(c->vcmd<(cmdVPortaD+10)))
-		c->fpitch=c->pitch=rangepitch(this, c->pitch+c->porta*16);
-
 	switch (c->command)
 	{
 		case cmdSpeed:
@@ -1025,19 +1121,19 @@ static void processfx(struct itplayer *this, struct it_logchan *c)
 			c->vol=range64(c->vol+tickslide(c->volslide));
 			break;
 		case cmdPortaD:
-			if (c->porta>=0xE0)
+			if (c->eporta>=0xE0)
 				break;
-			c->fpitch=c->pitch=rangepitch(this, c->pitch+c->porta*16);
+			c->fpitch=c->pitch=rangepitch(this, c->pitch+c->eporta*16);
 			break;
 		case cmdPortaU:
-			if (c->porta>=0xE0)
+			if (c->eporta>=0xE0)
 				break;
-			c->fpitch=c->pitch=rangepitch(this, c->pitch-c->porta*16);
+			c->fpitch=c->pitch=rangepitch(this, c->pitch-c->eporta*16);
 			if ((c->pitch==this->pitchhigh)&&c->pch)
 				c->pch->notecut=1;
 			break;
 		case cmdPortaNote:
-			doportanote(this, c);
+			doportanote(this, c, 0);
 			break;
 		case cmdVibrato:
 			dovibrato(this, c);
@@ -1068,7 +1164,7 @@ static void processfx(struct itplayer *this, struct it_logchan *c)
 				break;
 			}
 		case cmdPortaVol:
-			doportanote(this, c);
+			doportanote(this, c, 0);
 			c->fvol=c->vol=range64(c->vol+tickslide(c->volslide));
 			break;
 		case cmdVibVol:
@@ -1113,7 +1209,7 @@ static void processfx(struct itplayer *this, struct it_logchan *c)
 				case cmdSPatLoop:
 					break;
 				case cmdSNoteCut:
-					if ((this->curtick>=c->specialdata)&&c->pch)
+					if ((this->curtick>=c->specialdata)&&c->pch&&(!this->patdelayrow))
 						c->pch->notecut=1;
 					break;
 				case cmdSNoteDelay:
@@ -1146,6 +1242,23 @@ static void processfx(struct itplayer *this, struct it_logchan *c)
 			dopanbrello(this, c);
 			break;
 	}
+
+
+	if ((c->vcmd>=cmdVVolSlD)&&(c->vcmd<(cmdVVolSlD+10)))
+		c->fvol=c->vol=range64(c->vol-c->vvolslide);
+	else if ((c->vcmd>=cmdVVolSlU)&&(c->vcmd<(cmdVVolSlU+10)))
+		c->fvol=c->vol=range64(c->vol+c->vvolslide);
+	else if ((c->vcmd>=cmdVVibrato)&&(c->vcmd<(cmdVVibrato+10)))
+		dovibrato(this, c);
+	else if ((c->vcmd>=cmdVPortaNote)&&(c->vcmd<(cmdVPortaNote+10)))
+		doportanote(this, c, 1);
+	else if ((c->vcmd>=cmdVPortaU)&&(c->vcmd<(cmdVPortaU+10)))
+	{
+		c->fpitch=c->pitch=rangepitch(this, c->pitch-c->vporta*16);
+		if ((c->pitch==this->pitchhigh)&&c->pch)
+			c->pch->notecut=1;
+	} else if ((c->vcmd>=cmdVPortaD)&&(c->vcmd<(cmdVPortaD+10)))
+		c->fpitch=c->pitch=rangepitch(this, c->pitch+c->vporta*16);
 }
 
 static void inittickchan(struct it_physchan *p)
@@ -1549,7 +1662,7 @@ static void playtick(struct itplayer *this)
 	}
 
 	if (this->curtick==(this->speed+this->patdelaytick))
-	{
+	{		
 		this->patdelaytick=0;
 		this->curtick=0;
 		for (i=0; i<this->nchan; i++)
@@ -1584,6 +1697,15 @@ static void playtick(struct itplayer *this)
 				this->gotoord=0;
 				if (!this->manualgoto && this->noloop)
 					this->looped=1;
+				for (i=0; i<this->nchan; i++)
+				{
+					struct it_logchan *c=&this->channels[i];
+					c->retrigspd=1;
+					c->tremoron=1;
+					c->tremoroff=1;
+					c->tremoroncounter=0;
+					c->tremoroffcounter=0;
+				}
 			}
 			while (this->orders[this->gotoord]==0xFFFF)
 				this->gotoord++;
@@ -1618,30 +1740,37 @@ static void playtick(struct itplayer *this)
 			}
 			return;
 		}
-		
+	
 		for (i=0; i<this->nchan; i++)
 			initrow(&this->channels[i]);
 		while (*this->patptr)
 		{
 			struct it_logchan *c=&this->channels[*this->patptr++-1];
+			int delay;
 			if ((this->patptr[3]==cmdSpecial)&&this->patptr[4])
 			{
 				c->specialcmd=this->patptr[4]>>4;
 				c->specialdata=this->patptr[4]&0xF;
 			}
 			if ((this->patptr[3]==cmdSpecial)&&(c->specialcmd==cmdSNoteDelay))
+			{
 				memcpy(c->delayed, this->patptr, /*sizeof(c->delayed)*/5); /* else we might read unknown unmapped memory */
-			else {
+				delay=1;
+			} else {
 				if (this->patptr[0]||this->patptr[1])
 					playnote(this, c, this->patptr);
-				if (this->patptr[2])
-					playvcmd(this, c, this->patptr[2]);
+				delay=0;
 			}
 			if (this->patptr[3])
-				playcmd(this, c, this->patptr[3], this->patptr[4]);
+				playcmd(this, c, this->patptr[3], this->patptr[4]); /* we need notedata to know if Portamento should do anything */
+			if (!delay)
+				if (this->patptr[2])
+					playvcmd(this, c, this->patptr[2]);
 			this->patptr+=5;
 		}
 		this->patptr++;
+		if (this->patdelayrow) /* in case, the first SEx command was SE0, we fetch this by storing SEx+1 to this->patdelayrow */
+			this->patdelayrow--;
 	} else
 		for (i=0; i<this->nchan; i++)
 			processfx(this, &this->channels[i]);
@@ -1763,6 +1892,11 @@ int __attribute__ ((visibility ("internal"))) play(struct itplayer *this, const
 		c->cpan=m->inipan[i]&127;
 		c->srnd=c->cpan==100;
 		c->disabled=m->inipan[i]&128;
+		c->retrigspd=1;
+		c->tremoron=1;
+		c->tremoroff=1;
+		c->tremoroncounter=0;
+		c->tremoroffcounter=0;
 	}
 
 	if (!mcpOpenPlayer(ch, playtickstatic))
