@@ -1,9 +1,23 @@
 /* OpenCP Module Player
- * copyright (c) '07-'09 Stian Skjelstad <stian@nixia.no>
+ * copyright (c) '07-'10 Stian Skjelstad <stian@nixia.no>
  *
  * Handle charset conversion and error-recovery (from fancy UTF-stuff into
  * simplified cp437) using iconv(). Also must work around a known bug in the
  * glibc implementation.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
  
 #include "config.h"
@@ -615,7 +629,7 @@ void  __attribute__((constructor)) id3v2_charset_init(void)
 {
 	const char *temp;
 
-	if (temp = getenv("CHARSET"))
+	if ((temp = getenv("CHARSET")))
 		TOCODE = strdup(temp);
 	else
 		TOCODE = strdup("CP437");
