@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * MPPlay interface routines
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -ss040910   Stian Skjelstad <stian@nixia.no>
  *    -first release
@@ -33,6 +47,8 @@
 #define _MAX_FNAME 8
 #define _MAX_EXT 4
 
+/* #define MPEG_DEBUG 1 */
+
 static FILE *mpegfile;
 static uint32_t mpeglen;
 static uint32_t mpegrate;
@@ -509,18 +525,28 @@ static int mpegOpenFile(const char *path, struct moduleinfostruct *info, FILE *m
 		fseek(mpegfile, 0, SEEK_SET);
 		if (!memcmp(sig, "RIFF", 4))
 		{
+#ifdef MPEG_DEBUG
+			fprintf(stderr, "[mppplay.c]: container RIFF\n");
+#endif
+
 			fseek(mpegfile, 12, SEEK_SET);
 			fl=0;
 			while (1)
 			{
 				if (fread(sig, 1, 4, mpegfile)!=4)
 					break;
-				if (fread(&fl, 1, sizeof(uint32_t),  mpegfile)!=1)
+#ifdef MPEG_DEBUG
+				fprintf(stderr, "[mppplay.c]: chunk: %c%c%c%c\n", sig[0], sig[1], sig[2], sig[3]);
+#endif
+				if (fread(&fl, sizeof(uint32_t), 1, mpegfile)!=1)
 				{
 					fprintf(stderr, __FILE__ ": fread failed #3\n");
 					return errFileRead;
 				}
 				fl = uint32_little (fl);
+#ifdef MPEG_DEBUG
+				fprintf(stderr, "[mppplay.c]: length: %d\n", (int)fl);
+#endif
 				if (!memcmp(sig, "data", 4))
 				{
 					ofs=ftell(mpegfile);
@@ -582,4 +608,4 @@ static int mpegOpenFile(const char *path, struct moduleinfostruct *info, FILE *m
 }
 
 struct cpifaceplayerstruct mpegPlayer = {mpegOpenFile, mpegCloseFile};
-struct linkinfostruct dllextinfo = {"playmp2", "OpenCP Audio MPEG Player (c) 1994-04 Stian Skjelstad, Niklas Beisert & Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
+struct linkinfostruct dllextinfo = {"playmp2", "OpenCP Audio MPEG Player (c) 1994-09 Stian Skjelstad, Niklas Beisert & Tammo Hinrichs", DLLVERSION, 0 LINKINFOSTRUCT_NOEVENTS};
