@@ -1,9 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '05-'09 Stian Skjelstad <stian@nixia.no>
+ * copyright (c) '05-'10 Stian Skjelstad <stian@nixia.no>
  *
  * OPLPlay - Player for AdPlug - Replayer for many OPL2/OPL3 audio file formats.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
- 
 
 #include "config.h"
 #include <cstdlib>
@@ -297,8 +310,8 @@ static void oplIdler(void)
 			int len=read>>2;
 			while (len)
 			{
-				rs=int16_little(base[0]);
-				ls=int16_little(base[1]);
+				rs=/*int16_little(*/base[0]/*)*/;
+				ls=/*int16_little(*/base[1]/*)*/;
 				PANPROC;
 				if (srnd)
 					rs^=~0;
@@ -348,7 +361,7 @@ void oplIdle(void)
 		quietlen=bufdelta;
 	else /*if (sizeof(oplbuf)!=opllen)*/ /* EOF of the oplstream? */
 	{           /* should not the crap below match up easy with imuldiv(opllen>>2, 65536, plrRate) ??? TODO */
-		uint32_t towrap=imuldiv((((sizeof(oplbuf)+oplbufread-oplbufpos-1)%sizeof(oplbuf))>>(1 /* we are always given stereo */ + 1 /* we are always given 16bit */)), 65536, plrRate);
+		uint32_t towrap=imuldiv((((sizeof(oplbuf)+oplbufread-oplbufpos-1)%sizeof(oplbuf))>>(1 /* we are always given stereo */ + 1 /* we are always given 16bit */)), 65536, oplbufrate);
 		if (bufdelta>towrap)
 		{	/* will the eof hit inside the delta? */
 			/*quietlen=bufdelta-towrap;*/
