@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * XMPlay .MOD module loader
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *    -first release
@@ -424,7 +438,7 @@ static int loadmod(struct xmodule *m, FILE *file, int chan, int sig, int opt)
 		if (!sip->ptr)
 			return errAllocMem;
 		if ((result=fread(sip->ptr, 1, sip->length, file))!=sip->length)
-			fprintf(stderr, "xmlmod.c: warning: fread() failed #8 (%d of %d)\n", result, (unsigned int)sip->length);
+			fprintf(stderr, "xmlmod.c: warning: fread() failed #8 (%d of %d)\n", (int)result, (unsigned int)sip->length);
 		sp->handle=i;
 	}
 
