@@ -3,7 +3,7 @@
 
 #include <sys/time.h>
 
-#define DOS_CLK_TCK 1000000
+#define DOS_CLK_TCK 0x10000
 
 extern time_t dos_clock(void);
 extern void _splitpath(const char *src, char *drive, char *path, char *file, char *ext);
