@@ -32,7 +32,7 @@ static inline int32_t imuldiv(int32_t a,int32_t b,int32_t c)
 {
 #ifdef __PIC__
 	int d0, d1;
-	int32_t register retval;
+	register int32_t retval;
 	__asm__ __volatile__(
 		"pushl %%ebx\n"
 		"movl %%ecx, %%ebx\n"
@@ -45,7 +45,7 @@ static inline int32_t imuldiv(int32_t a,int32_t b,int32_t c)
 	return retval;
 #else
 	int d0, d1;
-	int32_t register retval;
+	register int32_t retval;
 	__asm__ __volatile__(
 		"imul %%edx\n"
 		"idiv %%ebx\n"
@@ -60,7 +60,7 @@ static inline uint32_t umuldiv(uint32_t a,uint32_t b,uint32_t c)
 {
 #ifdef __PIC__
 	int d0, d1;
-	uint32_t register retval;
+	register uint32_t retval;
 	__asm__ __volatile__
 	(
 	 	"pushl %%ebx\n"
@@ -74,7 +74,7 @@ static inline uint32_t umuldiv(uint32_t a,uint32_t b,uint32_t c)
 	return retval;
 #else
 	int d0, d1;
-	uint32_t register retval;
+	register uint32_t retval;
 	__asm__ __volatile__
 	(
 		"mul %%edx\n"
@@ -89,7 +89,7 @@ static inline uint32_t umuldiv(uint32_t a,uint32_t b,uint32_t c)
 static inline int32_t imulshr16(int32_t a,int32_t b)
 {
 	int d0;
-	int32_t register retval;
+	register int32_t retval;
 	__asm__ __volatile__
 	(
 		"imul %%edx\n"
@@ -103,7 +103,7 @@ static inline int32_t imulshr16(int32_t a,int32_t b)
 static inline uint32_t umulshr16(uint32_t a,uint32_t b)
 {
 	int d0;
-	uint32_t register retval;
+	register uint32_t retval;
 	__asm__ __volatile__
 	(
 		"mul %%edx\n"
@@ -117,7 +117,7 @@ static inline uint32_t umulshr16(uint32_t a,uint32_t b)
 static inline uint32_t umldivrnd(uint32_t a,uint32_t b,uint32_t c)
 {
 	int d0, d1;
-	uint32_t register retval;
+	register uint32_t retval;
 	__asm__ __volatile__
 	(
 		"mul %%edx\n"
