@@ -1,8 +1,22 @@
 /* OpenCP Module Player
- * copyright (c) '94-'05 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
+ * copyright (c) '94-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  *
  * X11 graphic driver
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  * revision history: (please note changes here)
  *  -ss050424   Stian Skjelstad <stian@nixia.no>
  *    -first release
@@ -51,8 +65,8 @@ typedef enum {
 	_FONT_MAX = 2
 } FontSizeEnum;
 
-FontSizeEnum plUseFont;
-FontSizeEnum plCurrentFont;
+static FontSizeEnum plUseFont = _8x8;
+static FontSizeEnum plCurrentFont = _8x8;
 
 static unsigned short plScrRowBytes;
 
@@ -466,12 +480,14 @@ static void set_state_graphmode(int target)
 		SizeHints.min_height=SizeHints.max_height=Graphmode_modeline->vdisplay;
 		XSetWMNormalHints(mDisplay, window, &SizeHints);
 		XResizeWindow(mDisplay, window, Graphmode_modeline->hdisplay, Graphmode_modeline->vdisplay);
-		XClearWindow(mDisplay, window);
+		XSync(mDisplay, FALSE);
+		XClearWindow(mDisplay, window);		
 	} else {
 		SizeHints.min_width=SizeHints.max_width=plScrLineBytes;
 		SizeHints.min_height=SizeHints.max_height=plScrLines;
 		XSetWMNormalHints(mDisplay, window, &SizeHints);
 		XResizeWindow(mDisplay, window, plScrLineBytes, plScrLines);
+		XSync(mDisplay, FALSE);
 	}
 	___push_key(VIRT_KEY_RESIZE);
 
@@ -562,10 +578,12 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 	if (do_fullscreen)
 	{
 		XResizeWindow(mDisplay, window, default_modeline.hdisplay, default_modeline.vdisplay);
+		XSync(mDisplay, FALSE);
 		plScrLines=default_modeline.vdisplay;
 		plScrLineBytes=default_modeline.hdisplay;
 	} else {
 		XResizeWindow(mDisplay, window, Textmode_Window_Width, Textmode_Window_Height);
+		XSync(mDisplay, FALSE);
 		plScrLines=Textmode_Window_Height;
 		plScrLineBytes=Textmode_Window_Width;
 	}
@@ -773,6 +791,7 @@ static void x11_common_event_loop(void)
 						case XK_Left:      key=KEY_CTRL_LEFT;   break;
 						case XK_Page_Up:   key=KEY_CTRL_PGUP;   break;
 						case XK_Page_Down: key=KEY_CTRL_PGDN;   break;	
+						case XK_Home:      key=KEY_CTRL_HOME;   break;
 					}
 				} else if ((event.xkey.state&(ControlMask|Mod1Mask))==Mod1Mask)
 				{	
@@ -881,10 +900,10 @@ static void create_window(void)
 
 static void create_image(void)
 {
-	if (XShmQueryExtension(mDisplay) )
+	if (mLocalDisplay && XShmQueryExtension(mDisplay) )
 	{
 		shm_completiontype = XShmGetEventBase(mDisplay) + ShmCompletion;
-		if (!(image = XShmCreateImage(mDisplay, XDefaultVisual(mDisplay, mScreen), plDepth, ZPixmap, NULL, &shminfo[0], plScrLineBytes, plScrLines)))
+		if (!(image = XShmCreateImage(mDisplay, XDefaultVisual(mDisplay, mScreen), XDefaultDepth(mDisplay, mScreen), ZPixmap, NULL, &shminfo[0], plScrLineBytes, plScrLines)))
 		{
 			fprintf(stderr, "[x11/shm] Failed to create XShmImage object\n");
 			exit(-1);
@@ -916,6 +935,7 @@ static void create_image(void)
 			exit(-1);
 		}
 	}
+	plDepth = image->bits_per_pixel;
 }
 
 static void destroy_image(void)
@@ -1335,14 +1355,30 @@ static void RefreshScreenGraph(void)
 		uint8_t *dst_line = (uint8_t *)image->data;
 		int Y=0, j;
 
-		if ((plDepth==32)||(plDepth==24))
+		if (plDepth==32)
 		{ 
 			uint32_t *dst;
 			while (1)
 			{
 				dst = (uint32_t *)dst_line;
 				for (j=0;j<plScrLineBytes;j++)
-					*(dst++)=palette32[*(src++)];
+					*(dst++)=x11_palette32[*(src++)];
+				if ((++Y)>=plScrLines)
+					break;
+				dst_line += image->bytes_per_line;
+			}
+		} else if (plDepth==24)
+		{ 
+			uint8_t *dst;
+			while (1)
+			{
+				dst = (uint8_t *)dst_line;
+				for (j=0;j<plScrLineBytes;j++)
+				{
+					*(dst++)=x11_palette32[*src]&255;
+					*(dst++)=(x11_palette32[*src]>>8) & 255;
+					*(dst++)=(x11_palette32[*src++]>>16) & 255;
+				}
 				if ((++Y)>=plScrLines)
 					break;
 				dst_line += image->bytes_per_line;
@@ -1354,7 +1390,7 @@ static void RefreshScreenGraph(void)
 			{
 				dst = (uint16_t *)dst_line;
 				for (j=0;j<plScrLineBytes;j++)
-					*(dst++)=palette16[*(src++)];
+					*(dst++)=x11_palette16[*(src++)];
 				if ((++Y)>=plScrLines)
 					break;
 				dst_line += image->bytes_per_line;
@@ -1366,7 +1402,7 @@ static void RefreshScreenGraph(void)
 			{
 				dst = (uint16_t *)dst_line;
 				for (j=0;j<plScrLineBytes;j++)
-					*(dst++)=palette15[*(src++)];
+					*(dst++)=x11_palette15[*(src++)];
 				if ((++Y)>=plScrLines)
 					break;
 				dst_line += image->bytes_per_line;
@@ -1581,8 +1617,8 @@ static void RefreshScreenText(void)
 						int i, j;
 						cp=plFont816[*(mem++)];
 						a=*(mem++);
-						f=palette15[a&15];
-						b=palette15[a>>4];
+						f=x11_palette15[a&15];
+						b=x11_palette15[a>>4];
 						for (i=0; i<16; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1622,8 +1658,8 @@ static void RefreshScreenText(void)
 						int i, j;
 						cp=plFont88[*(mem++)];
 						a=*(mem++);
-						f=palette15[a&15];
-						b=palette15[a>>4];
+						f=x11_palette15[a&15];
+						b=x11_palette15[a>>4];
 						for (i=0; i<8; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1663,8 +1699,8 @@ static void RefreshScreenText(void)
 						int i, j;
 						cp=plFont44[*(mem++)];
 						a=*(mem++);
-						f=palette15[a&15];
-						b=palette15[a>>4];
+						f=x11_palette15[a&15];
+						b=x11_palette15[a>>4];
 						for (i=0; i<2; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1729,8 +1765,8 @@ static void RefreshScreenText(void)
 
 						cp=plFont816[*(mem++)];
 						a=*(mem++);
-						f=palette16[a&15];
-						b=palette16[a>>4];
+						f=x11_palette16[a&15];
+						b=x11_palette16[a>>4];
 						for (i=0; i<16; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1776,8 +1812,8 @@ static void RefreshScreenText(void)
 
 						cp=plFont88[*(mem++)];
 						a=*(mem++);
-						f=palette16[a&15];
-						b=palette16[a>>4];
+						f=x11_palette16[a&15];
+						b=x11_palette16[a>>4];
 						for (i=0; i<8; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1823,8 +1859,8 @@ static void RefreshScreenText(void)
 
 						cp=plFont44[*(mem++)];
 						a=*(mem++);
-						f=palette16[a&15];
-						b=palette16[a>>4];
+						f=x11_palette16[a&15];
+						b=x11_palette16[a>>4];
 						for (i=0; i<2; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1874,6 +1910,232 @@ static void RefreshScreenText(void)
 			}
 			break;
 		case 24:
+#if 0
+			switch (plCurrentFont)
+			{
+			case _8x16:
+				for (y=0;y<plScrHeight;y++)
+				{
+					uint8_t *scr_precalc = ((uint8_t *)image->data)+y*16*image->bytes_per_line;
+					for (x=0;x<plScrWidth;x++)
+					{
+						uint8_t f1, f2, f3, b1, b2, b3;
+						int i, j;
+						uint8_t a, *cp;
+						uint8_t *scr = scr_precalc;
+						scr_precalc += 24;
+
+						cp=plFont816[*(mem++)];
+						a=*(mem++);
+						f1=x11_palette32[a&15] & 255;
+						f2=(x11_palette32[a&15]>>8) & 255;
+						f3=(x11_palette32[a&15]>>16) & 255;
+						b1=x11_palette32[a>>4] & 255;
+						b2=(x11_palette32[a>>4]>>8) & 255;
+						b3=(x11_palette32[a>>4]>>16) & 255;
+
+						for (i=0; i<16; i++)
+						{
+							uint8_t bitmap=*cp++;
+							for (j=0; j<8; j++)
+							{
+								if (bitmap&128)
+								{
+									*scr++=f1;
+									*scr++=f2;
+									*scr++=f3;
+								} else {
+									*scr++=b1;
+									*scr++=b2;
+									*scr++=b3;
+								}
+								bitmap<<=1;
+							}
+							scr += image->bytes_per_line - 24;
+						}
+						if ((doshape==1)&&(curposy==y)&&(curposx==x))
+						{
+							cp=plFont816['_']+15;
+							scr+=24;
+							for (i=0; i<16; i++)
+							{
+								uint8_t bitmap=*cp--;
+								scr -= image->bytes_per_line + 24;
+								for (j=0; j<8; j++)
+								{
+									if (bitmap&1)
+									{
+										scr[0]=f1;
+										scr[1]=f2;
+										scr[2]=f3;
+									}
+									bitmap>>=1;
+									scr+=3;
+								}
+							}
+						}
+					}
+				}
+				break;
+			case _8x8:
+				for (y=0;y<plScrHeight;y++)
+				{
+					uint8_t *scr_precalc = ((uint8_t *)image->data)+y*8*image->bytes_per_line;
+					for (x=0;x<plScrWidth;x++)
+					{
+						uint8_t f1, f2, f3, b1, b2, b3;
+						int i, j;
+						uint8_t a, *cp;
+						uint8_t *scr=scr_precalc;
+						scr_precalc+=24;
+						cp=plFont88[*(mem++)];
+						a=*(mem++);
+						f1=x11_palette32[a&15] & 255;
+						f2=(x11_palette32[a&15]>>8) & 255;
+						f3=(x11_palette32[a&15]>>16) & 255;
+						b1=x11_palette32[a>>4] & 255;
+						b2=(x11_palette32[a>>4]>>8) & 255;
+						b3=(x11_palette32[a>>4]>>16) & 255;
+
+						for (i=0; i<8; i++)
+						{
+							uint8_t bitmap=*cp++;
+							for (j=0; j<8; j++)
+							{
+								if (bitmap&128)
+								{
+									*scr++=f1;
+									*scr++=f2;
+									*scr++=f3;
+								} else {
+									*scr++=b1;
+									*scr++=b2;
+									*scr++=b3;
+								}
+								bitmap<<=1;
+							}
+							scr += image->bytes_per_line - 24;
+						}
+						if ((doshape==1)&&(curposy==y)&&(curposx==x))
+						{
+							cp=plFont88['_']+7;
+							scr+=24;
+							for (i=0; i<8; i++)
+							{
+								uint8_t bitmap=*cp--;
+								scr -= image->bytes_per_line + 24;
+								for (j=0; j<8; j++)
+								{
+									if (bitmap&1)
+									{
+										scr[0]=f1;
+										scr[1]=f2;
+										scr[2]=f3;
+									}
+									bitmap>>=1;
+									scr+=3;
+								}
+							}
+						}
+					}
+				}
+				break;
+			case _4x4:
+				for (y=0;y<plScrHeight;y++)
+				{
+					uint8_t *scr_precalc = ((uint8_t *)image->data)+y*4*image->bytes_per_line;
+					for (x=0;x<plScrWidth;x++)
+					{
+						uint8_t f1, f2, f3, b1, b2, b3;
+						int i, j;
+						uint8_t a, *cp;
+						uint8_t *scr=scr_precalc;
+						scr_precalc+=12;
+						cp=plFont44[*(mem++)];
+						a=*(mem++);
+						f1=x11_palette32[a&15] & 255;
+						f2=(x11_palette32[a&15]>>8) & 255;
+						f3=(x11_palette32[a&15]>>16) & 255;
+						b1=x11_palette32[a>>4] & 255;
+						b2=(x11_palette32[a>>4]>>8) & 255;
+						b3=(x11_palette32[a>>4]>>16) & 255;
+
+						for (i=0; i<2; i++)
+						{
+							uint8_t bitmap = *cp++;
+
+							for (j=0; j<4; j++)
+							{
+								if (bitmap&128)
+								{
+									*scr++=f1;
+									*scr++=f2;
+									*scr++=f3;
+								} else {
+									*scr++=b1;
+									*scr++=b2;
+									*scr++=b3;
+								}
+								bitmap<<=1;
+							}
+							scr += image->bytes_per_line - 12;
+
+							for (j=0; j<4; j++)
+							{
+								if (bitmap&128)
+								{
+									*scr++=f1;
+									*scr++=f2;
+									*scr++=f3;
+								} else {
+									*scr++=b1;
+									*scr++=b2;
+									*scr++=b3;
+								}
+								bitmap<<=1;
+							}
+							scr += image->bytes_per_line - 12;
+
+						}
+						if ((doshape==1)&&(curposy==y)&&(curposx==x))
+						{
+							cp=plFont44['_']+1;
+							scr+=12;
+							for (i=0; i<2; i++)
+							{
+								uint8_t bitmap=*cp--;
+								scr -= image->bytes_per_line + 12;
+								for (j=0; j<4; j++)
+								{
+									if (bitmap&1)
+									{
+										scr[0]=f1;
+										scr[1]=f2;
+										scr[2]=f3;
+									}
+									bitmap>>=1;
+									scr+=3;
+								}
+								scr -= image->bytes_per_line + 12;
+								for (j=0; j<4; j++)
+								{
+									if (bitmap&1)
+									{
+										scr[0]=f1;
+										scr[1]=f2;
+										scr[2]=f3;
+									}
+									bitmap>>=1;
+									scr+=3;
+								}
+							}
+						}
+					}
+				}
+				break;
+			}
+			break;
+#endif
 		case 32:
 			precalc_linelength = image->bytes_per_line/sizeof(uint32_t);
 			switch (plCurrentFont)
@@ -1892,8 +2154,8 @@ static void RefreshScreenText(void)
 
 						cp=plFont816[*(mem++)];
 						a=*(mem++);
-						f=palette32[a&15];
-						b=palette32[a>>4];
+						f=x11_palette32[a&15];
+						b=x11_palette32[a>>4];
 						for (i=0; i<16; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1937,8 +2199,8 @@ static void RefreshScreenText(void)
 						scr_precalc+=8;
 						cp=plFont88[*(mem++)];
 						a=*(mem++);
-						f=palette32[a&15];
-						b=palette32[a>>4];
+						f=x11_palette32[a&15];
+						b=x11_palette32[a>>4];
 						for (i=0; i<8; i++)
 						{
 							uint8_t bitmap=*cp++;
@@ -1982,8 +2244,8 @@ static void RefreshScreenText(void)
 						scr_precalc+=4;
 						cp=plFont44[*(mem++)];
 						a=*(mem++);
-						f=palette32[a&15];
-						b=palette32[a>>4];
+						f=x11_palette32[a&15];
+						b=x11_palette32[a>>4];
 						for (i=0; i<2; i++)
 						{
 							uint8_t bitmap = *cp++;
@@ -2261,6 +2523,7 @@ static int ___valid_key(uint16_t key)
 		case KEY_CTRL_PGUP:
 		case KEY_CTRL_PGDN:
 		case KEY_CTRL_ENTER:
+		case KEY_CTRL_HOME:
 			return 1;
 
 		default:
